@@ -1,37 +1,33 @@
 /*
- *  Copyright (C) 2004-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  Author: Guillaume Roguez <Guillaume.Roguez@savoirfairelinux.com>
- *  Author: Philippe Gorley <philippe.gorley@savoirfairelinux.com>
- *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "libav_deps.h" // MUST BE INCLUDED FIRST
 #include "media_decoder.h"
 #include "media_device.h"
 #include "media_buffer.h"
+#include "media_const.h"
 #include "media_io_handle.h"
-#include "audio/audiobuffer.h"
 #include "audio/ringbuffer.h"
 #include "audio/resampler.h"
+#include "audio/ringbufferpool.h"
 #include "decoder_finder.h"
 #include "manager.h"
 
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
 #include "video/accel.h"
 #endif
 
@@ -94,84 +90,108 @@ MediaDemuxer::openInput(const DeviceParams& params)
     auto iformat = av_find_input_format(params.format.c_str());
 
     if (!iformat && !params.format.empty())
-        JAMI_WARN("Cannot find format \"%s\"", params.format.c_str());
+        JAMI_WARN("Unable to find format \"%s\"", params.format.c_str());
 
-    if (params.width and params.height) {
-        auto sizeStr = fmt::format("{}x{}", params.width, params.height);
-        av_dict_set(&options_, "video_size", sizeStr.c_str(), 0);
-    }
+    std::string input;
 
-    if (params.framerate) {
+    if (params.input == "pipewiregrab") {
+        //
+        // We rely on pipewiregrab for screen/window sharing on Wayland.
+        // Because pipewiregrab is a "video source filter" (part of FFmpeg's libavfilter
+        // library), its options must all be passed as part of the `input` string.
+        //
+        input = fmt::format("pipewiregrab=draw_mouse=1:fd={}:node={}", params.fd, params.node);
+        JAMI_LOG("Attempting to open input {}", input);
+        //
+        // In all other cases, we use the `options_` AVDictionary to pass options to FFmpeg.
+        //
+        // NOTE: We rely on the "lavfi" virtual input device to read pipewiregrab's output
+        // and create a corresponding stream (cf. the getDeviceParams function in
+        // daemon/src/media/video/v4l2/video_device_impl.cpp). The `options_` dictionary
+        // could be used to set lavfi's parameters if that was ever needed, but it isn't at
+        // the moment. (Doc: https://ffmpeg.org/ffmpeg-devices.html#lavfi)
+        //
+    } else {
+        if (params.width and params.height) {
+            auto sizeStr = fmt::format("{}x{}", params.width, params.height);
+            av_dict_set(&options_, "video_size", sizeStr.c_str(), 0);
+        }
+
+        if (params.framerate) {
 #ifdef _WIN32
-        // On windows, framerate settings don't reduce to avrational values
-        // that correspond to valid video device formats.
-        // e.g. A the rational<double>(10000000, 333333) or 30.000030000
-        //      will be reduced by av_reduce to 999991/33333 or 30.00003000003
-        //      which cause the device opening routine to fail.
-        // So we treat this imprecise reduction and adjust the value,
-        // or let dshow choose the framerate, which is, unfortunately,
-        // NOT the highest according to our experimentations.
-        auto framerate {params.framerate.real()};
-        framerate = params.framerate.numerator() / (params.framerate.denominator() + 0.5);
-        if (params.framerate.denominator() != 4999998)
-            av_dict_set(&options_, "framerate", jami::to_string(framerate).c_str(), 0);
+            // On windows, framerate settings don't reduce to avrational values
+            // that correspond to valid video device formats.
+            // e.g. A the rational<double>(10000000, 333333) or 30.000030000
+            //      will be reduced by av_reduce to 999991/33333 or 30.00003000003
+            //      which cause the device opening routine to fail.
+            // So we treat this imprecise reduction and adjust the value,
+            // or let dshow choose the framerate, which is, unfortunately,
+            // NOT the highest according to our experimentations.
+            auto framerate {params.framerate.real()};
+            framerate = params.framerate.numerator() / (params.framerate.denominator() + 0.5);
+            if (params.framerate.denominator() != 4999998)
+                av_dict_set(&options_, "framerate", jami::to_string(framerate).c_str(), 0);
 #else
-        av_dict_set(&options_, "framerate", jami::to_string(params.framerate.real()).c_str(), 0);
+            av_dict_set(&options_, "framerate", jami::to_string(params.framerate.real()).c_str(), 0);
 #endif
-    }
+        }
 
-    if (params.offset_x || params.offset_y) {
-        av_dict_set(&options_, "offset_x", std::to_string(params.offset_x).c_str(), 0);
-        av_dict_set(&options_, "offset_y", std::to_string(params.offset_y).c_str(), 0);
-    }
-    if (params.channel)
-        av_dict_set(&options_, "channel", std::to_string(params.channel).c_str(), 0);
-    av_dict_set(&options_, "loop", params.loop.c_str(), 0);
-    av_dict_set(&options_, "sdp_flags", params.sdp_flags.c_str(), 0);
+        if (params.offset_x || params.offset_y) {
+            av_dict_set(&options_, "offset_x", std::to_string(params.offset_x).c_str(), 0);
+            av_dict_set(&options_, "offset_y", std::to_string(params.offset_y).c_str(), 0);
+        }
+        if (params.channel)
+            av_dict_set(&options_, "channel", std::to_string(params.channel).c_str(), 0);
+        av_dict_set(&options_, "loop", params.loop.c_str(), 0);
+        av_dict_set(&options_, "sdp_flags", params.sdp_flags.c_str(), 0);
 
-    // Set jitter buffer options
-    av_dict_set(&options_, "reorder_queue_size", std::to_string(jitterBufferMaxSize_).c_str(), 0);
-    auto us = std::chrono::duration_cast<std::chrono::microseconds>(jitterBufferMaxDelay_).count();
-    av_dict_set(&options_, "max_delay", std::to_string(us).c_str(), 0);
+        // Set jitter buffer options
+        av_dict_set(&options_, "reorder_queue_size", std::to_string(jitterBufferMaxSize_).c_str(), 0);
+        auto us = std::chrono::duration_cast<std::chrono::microseconds>(jitterBufferMaxDelay_).count();
+        av_dict_set(&options_, "max_delay", std::to_string(us).c_str(), 0);
 
-    if (!params.pixel_format.empty()) {
-        av_dict_set(&options_, "pixel_format", params.pixel_format.c_str(), 0);
-    }
-    if (!params.window_id.empty()) {
-        av_dict_set(&options_, "window_id", params.window_id.c_str(), 0);
-    }
-    av_dict_set(&options_, "is_area", std::to_string(params.is_area).c_str(), 0);
+        if (!params.pixel_format.empty()) {
+            av_dict_set(&options_, "pixel_format", params.pixel_format.c_str(), 0);
+        }
+        if (!params.window_id.empty()) {
+            av_dict_set(&options_, "window_id", params.window_id.c_str(), 0);
+        }
+        av_dict_set(&options_, "draw_mouse", "1", 0);
+        av_dict_set(&options_, "is_area", std::to_string(params.is_area).c_str(), 0);
 
 #if defined(__APPLE__) && TARGET_OS_MAC
-    std::string input = params.name;
+        input = params.name;
 #else
-    std::string input = params.input;
+        input = params.input;
 #endif
 
-    JAMI_DBG("Trying to open device %s with format %s, pixel format %s, size %dx%d, rate %lf",
-             input.c_str(),
-             params.format.c_str(),
-             params.pixel_format.c_str(),
-             params.width,
-             params.height,
-             params.framerate.real());
+        JAMI_LOG("Attempting to open input {} with format {}, pixel format {}, size {}x{}, rate {}",
+                 input,
+                 params.format,
+                 params.pixel_format,
+                 params.width,
+                 params.height,
+                 params.framerate.real());
+    }
+
+    // Ask FFmpeg to open the input using the options set above
+    if (params.disable_dts_probe_delay && params.format == "sdp") {
+        av_opt_set_int(inputCtx_, "max_ts_probe", 0, AV_OPT_SEARCH_CHILDREN);
+        av_opt_set_int(inputCtx_, "fpsprobesize", 0, AV_OPT_SEARCH_CHILDREN);
+    } else {
+        // Don't waste time fetching framerate when finding stream info
+        av_opt_set_int(inputCtx_, "fpsprobesize", 1, AV_OPT_SEARCH_CHILDREN);
+    }
 
-    av_opt_set_int(
-        inputCtx_,
-        "fpsprobesize",
-        1,
-        AV_OPT_SEARCH_CHILDREN); // Don't waste time fetching framerate when finding stream info
     int ret = avformat_open_input(&inputCtx_, input.c_str(), iformat, options_ ? &options_ : NULL);
 
     if (ret) {
-        JAMI_ERR("avformat_open_input failed: %s", libav_utils::getError(ret).c_str());
-    } else {
+        JAMI_ERROR("avformat_open_input failed: {}", libav_utils::getError(ret));
+    } else if (inputCtx_->nb_streams > 0 && inputCtx_->streams[0]->codecpar) {
         baseWidth_ = inputCtx_->streams[0]->codecpar->width;
         baseHeight_ = inputCtx_->streams[0]->codecpar->height;
-        JAMI_DBG("Using format %s and resolution %dx%d",
-                 params.format.c_str(),
-                 baseWidth_,
-                 baseHeight_);
+        JAMI_LOG("Opened input Using format {:s} and resolution {:d}x{:d}",
+                 params.format, baseWidth_, baseHeight_);
     }
 
     return ret;
@@ -200,7 +220,7 @@ MediaDemuxer::findStreamInfo()
         inputCtx_->max_analyze_duration = 30 * AV_TIME_BASE;
         int err;
         if ((err = avformat_find_stream_info(inputCtx_, nullptr)) < 0) {
-            JAMI_ERR() << "Could not find stream info: " << libav_utils::getError(err);
+            JAMI_ERR() << "Unable to find stream info: " << libav_utils::getError(err);
         }
         streamInfoFound_ = true;
     }
@@ -209,7 +229,16 @@ MediaDemuxer::findStreamInfo()
 int
 MediaDemuxer::selectStream(AVMediaType type)
 {
-    return av_find_best_stream(inputCtx_, type, -1, -1, nullptr, 0);
+    auto sti = av_find_best_stream(inputCtx_, type, -1, -1, nullptr, 0);
+    if (type == AVMEDIA_TYPE_VIDEO && sti >= 0) {
+        auto st = inputCtx_->streams[sti];
+        auto disposition = st->disposition;
+        if (disposition & AV_DISPOSITION_ATTACHED_PIC) {
+            JAMI_DBG("Skipping attached picture stream");
+            sti = -1;
+        }
+    }
+    return sti;
 }
 
 void
@@ -238,59 +267,58 @@ void
 MediaDemuxer::clearFrames()
 {
     {
-        std::lock_guard<std::mutex> lk {videoBufferMutex_};
+        std::lock_guard lk {videoBufferMutex_};
         while (!videoBuffer_.empty()) {
             videoBuffer_.pop();
         }
     }
     {
-        std::lock_guard<std::mutex> lk {audioBufferMutex_};
+        std::lock_guard lk {audioBufferMutex_};
         while (!audioBuffer_.empty()) {
             audioBuffer_.pop();
         }
     }
 }
 
-void
+bool
 MediaDemuxer::emitFrame(bool isAudio)
 {
     if (isAudio) {
-        pushFrameFrom(audioBuffer_, isAudio, audioBufferMutex_);
+        return pushFrameFrom(audioBuffer_, isAudio, audioBufferMutex_);
     } else {
-        pushFrameFrom(videoBuffer_, isAudio, videoBufferMutex_);
+        return pushFrameFrom(videoBuffer_, isAudio, videoBufferMutex_);
     }
 }
 
-void
+bool
 MediaDemuxer::pushFrameFrom(
     std::queue<std::unique_ptr<AVPacket, std::function<void(AVPacket*)>>>& buffer,
     bool isAudio,
     std::mutex& mutex)
 {
-    std::unique_lock<std::mutex> lock(mutex);
+    std::unique_lock lock(mutex);
     if (buffer.empty()) {
         if (currentState_ == MediaDemuxer::CurrentState::Finished) {
             fileFinishedCb_(isAudio);
         } else {
             needFrameCb_();
         }
-        return;
+        return false;
     }
     auto packet = std::move(buffer.front());
     if (!packet) {
-        return;
+        return false;
     }
     auto streamIndex = packet->stream_index;
     if (static_cast<unsigned>(streamIndex) >= streams_.size() || streamIndex < 0) {
-        return;
+        return false;
     }
-    auto& cb = streams_[streamIndex];
-    if (!cb) {
-        return;
+    if (auto& cb = streams_[streamIndex]) {
+        buffer.pop();
+        lock.unlock();
+        cb(*packet.get());
     }
-    buffer.pop();
-    lock.unlock();
-    cb(*packet.get());
+    return true;
 }
 
 MediaDemuxer::Status
@@ -309,7 +337,7 @@ MediaDemuxer::demuxe()
     } else if (ret == AVERROR_EOF) {
         return Status::EndOfFile;
     } else if (ret < 0) {
-        JAMI_ERR("Couldn't read frame: %s\n", libav_utils::getError(ret).c_str());
+        JAMI_ERR("Unable to read frame: %s\n", libav_utils::getError(ret).c_str());
         return Status::ReadError;
     }
 
@@ -320,13 +348,13 @@ MediaDemuxer::demuxe()
 
     AVStream* stream = inputCtx_->streams[streamIndex];
     if (stream->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
-        std::lock_guard<std::mutex> lk {videoBufferMutex_};
+        std::lock_guard lk {videoBufferMutex_};
         videoBuffer_.push(std::move(packet));
         if (videoBuffer_.size() >= 90) {
             return Status::ReadBufferOverflow;
         }
     } else {
-        std::lock_guard<std::mutex> lk {audioBufferMutex_};
+        std::lock_guard lk {audioBufferMutex_};
         audioBuffer_.push(std::move(packet));
         if (audioBuffer_.size() >= 300) {
             return Status::ReadBufferOverflow;
@@ -347,7 +375,7 @@ MediaDemuxer::decode()
     if (inputParams_.format == "x11grab" || inputParams_.format == "dxgigrab") {
         auto ret = inputCtx_->iformat->read_header(inputCtx_);
         if (ret == AVERROR_EXTERNAL) {
-            JAMI_ERR("Couldn't read frame: %s\n", libav_utils::getError(ret).c_str());
+            JAMI_ERR("Unable to read frame: %s\n", libav_utils::getError(ret).c_str());
             return Status::ReadError;
         }
         auto codecpar = inputCtx_->streams[0]->codecpar;
@@ -388,7 +416,7 @@ MediaDemuxer::decode()
         const auto type = media == AVMediaType::AVMEDIA_TYPE_AUDIO
                               ? "AUDIO"
                               : (media == AVMediaType::AVMEDIA_TYPE_VIDEO ? "VIDEO" : "UNSUPPORTED");
-        JAMI_ERR("Couldn't read [%s] frame: %s\n", type, libav_utils::getError(ret).c_str());
+        JAMI_ERR("Unable to read [%s] frame: %s\n", type, libav_utils::getError(ret).c_str());
         return Status::ReadError;
     }
 
@@ -427,10 +455,10 @@ MediaDecoder::MediaDecoder(const std::shared_ptr<MediaDemuxer>& demuxer,
     setupStream();
 }
 
-void
+bool
 MediaDecoder::emitFrame(bool isAudio)
 {
-    demuxer_->emitFrame(isAudio);
+    return demuxer_->emitFrame(isAudio);
 }
 
 MediaDecoder::MediaDecoder()
@@ -444,7 +472,7 @@ MediaDecoder::MediaDecoder(MediaObserver o)
 
 MediaDecoder::~MediaDecoder()
 {
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (decoderCtx_ && decoderCtx_->hw_device_ctx)
         av_buffer_unref(&decoderCtx_->hw_device_ctx);
 #endif
@@ -503,7 +531,7 @@ MediaDecoder::setupStream()
     if (prepareDecoderContext() < 0)
         return -1; // failed
 
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     // if there was a fallback to software decoding, do not enable accel
     // it has been disabled already by the video_receive_thread/video_input
     enableAccel_ &= Manager::instance().videoPreferences.getDecodingAccelerated();
@@ -554,7 +582,7 @@ MediaDecoder::setupStream()
         JAMI_DBG() << "Using framerate emulation";
     startTime_ = av_gettime(); // used to set pts after decoding, and for rate emulation
 
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (!accel_) {
         JAMI_WARN("Not using hardware decoding for %s", avcodec_get_name(decoderCtx_->codec_id));
         ret = avcodec_open2(decoderCtx_, inputDecoder_, nullptr);
@@ -563,7 +591,7 @@ MediaDecoder::setupStream()
     ret = avcodec_open2(decoderCtx_, inputDecoder_, nullptr);
 #endif
     if (ret < 0) {
-        JAMI_ERR() << "Could not open codec: " << libav_utils::getError(ret);
+        JAMI_ERR() << "Unable to open codec: " << libav_utils::getError(ret);
         return -1;
     }
 
@@ -575,13 +603,13 @@ MediaDecoder::prepareDecoderContext()
 {
     inputDecoder_ = findDecoder(avStream_->codecpar->codec_id);
     if (!inputDecoder_) {
-        JAMI_ERR() << "Unsupported codec";
+        JAMI_ERROR("Unsupported codec");
         return -1;
     }
 
     decoderCtx_ = avcodec_alloc_context3(inputDecoder_);
     if (!decoderCtx_) {
-        JAMI_ERR() << "Failed to create decoder context";
+        JAMI_ERROR("Failed to create decoder context");
         return -1;
     }
     avcodec_parameters_to_context(decoderCtx_, avStream_->codecpar);
@@ -594,10 +622,13 @@ MediaDecoder::prepareDecoderContext()
         if (decoderCtx_->framerate.num == 0 || decoderCtx_->framerate.den == 0)
             decoderCtx_->framerate = {30, 1};
     }
-    if (avStream_->codecpar->codec_type == AVMEDIA_TYPE_AUDIO) {
+    else if (avStream_->codecpar->codec_type == AVMEDIA_TYPE_AUDIO) {
         if (decoderCtx_->codec_id == AV_CODEC_ID_OPUS) {
             av_opt_set_int(decoderCtx_, "decode_fec", fecEnabled_ ? 1 : 0, AV_OPT_SEARCH_CHILDREN);
         }
+        auto format = libav_utils::choose_sample_fmt_default(inputDecoder_, Manager::instance().getRingBufferPool().getInternalAudioFormat().sampleFormat);
+        decoderCtx_->sample_fmt = format;
+        decoderCtx_->request_sample_fmt = format;
     }
     return 0;
 }
@@ -613,8 +644,16 @@ MediaDecoder::decode(AVPacket& packet)
 {
     int frameFinished = 0;
     auto ret = avcodec_send_packet(decoderCtx_, &packet);
+    // TODO: Investigate avcodec_send_packet returning AVERROR_INVALIDDATA.
+    // * Bug Windows documented here: git.jami.net/savoirfairelinux/jami-daemon/-/issues/1116
+    // where avcodec_send_packet returns AVERROR_INVALIDDATA when the size information in the
+    // packet is incorrect. Falling back onto sw decoding in this causes a segfault.
+    // * A second problem occurs on some Windows devices with intel CPUs in which hardware
+    // decoding fails with AVERROR_INVALIDDATA when using H.264. However, in this scenario,
+    // falling back to software decoding works fine.
+    // We need to figure out why this behavior occurs and how to discriminate between the two.
     if (ret < 0 && ret != AVERROR(EAGAIN)) {
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
         if (accel_) {
             JAMI_WARN("Decoding error falling back to software");
             fallback_ = true;
@@ -678,7 +717,7 @@ MediaDecoder::decode(AVPacket& packet)
         if (emulateRate_ and packetTimestamp != AV_NOPTS_VALUE) {
             auto startTime = avStream_->start_time == AV_NOPTS_VALUE ? 0 : avStream_->start_time;
             rational<double> frame_time = rational<double>(getTimeBase())
-                                          * (packetTimestamp - startTime);
+                                          * rational<double>(packetTimestamp - startTime);
             auto target_relative = static_cast<std::int64_t>(frame_time.real() * 1e6);
             auto target_absolute = startTime_ + target_relative;
             if (target_relative < seekTime_) {
@@ -725,7 +764,7 @@ MediaDecoder::decode()
 }
 
 #ifdef ENABLE_VIDEO
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
 void
 MediaDecoder::enableAccel(bool enableAccel)
 {
@@ -833,8 +872,12 @@ MediaDecoder::correctPixFmt(int input_pix_fmt)
 MediaStream
 MediaDecoder::getStream(std::string name) const
 {
+    if (!decoderCtx_) {
+        JAMI_WARN("No decoder context");
+        return {};
+    }
     auto ms = MediaStream(name, decoderCtx_, lastTimestamp_);
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     // accel_ is null if not using accelerated codecs
     if (accel_)
         ms.format = accel_->getSoftwareFormat();
