@@ -1,18 +1,18 @@
 /*
- *  Copyright (C) 2004-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3, or (at your option)
- *  any later version.
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, see <https://www.gnu.org/licenses/>.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 #include "libav_deps.h" // MUST BE INCLUDED FIRST
 #include "media_codec.h"
@@ -26,7 +26,7 @@
 #include "string_utils.h"
 #include "system_codec_container.h"
 
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
 #include "video/accel.h"
 #endif
 
@@ -167,7 +167,7 @@ MediaEncoder::openOutput(const std::string& filename, const std::string& format)
                                                 format.empty() ? nullptr : format.c_str(),
                                                 filename.c_str());
     if (result < 0)
-        JAMI_ERR() << "Cannot open " << filename << ": " << libav_utils::getError(-result);
+        JAMI_ERR() << "Unable to open " << filename << ": " << libav_utils::getError(-result);
 }
 
 int
@@ -225,15 +225,15 @@ MediaEncoder::initStream(const SystemCodecInfo& systemCodecInfo, AVBufferRef* fr
              systemCodecInfo.name.c_str(),
              systemCodecInfo.libName.c_str());
 
-    std::lock_guard<std::mutex> lk(encMutex_);
+    std::lock_guard lk(encMutex_);
 
     if (!outputCtx_)
-        throw MediaEncoderException("Cannot allocate stream");
+        throw MediaEncoderException("Unable to allocate stream");
 
     // Must already have codec instance(s)
     if (outputCtx_->nb_streams == 0) {
-        JAMI_ERR("[%p] Can not init, output context has no coding sessions!", this);
-        throw MediaEncoderException("Can not init, output context has no coding sessions!");
+        JAMI_ERR("[%p] Unable to init, output context has no coding sessions!", this);
+        throw MediaEncoderException("Unable to init, output context has no coding sessions!");
     }
 
     AVCodecContext* encoderCtx = nullptr;
@@ -261,14 +261,14 @@ MediaEncoder::initStream(const SystemCodecInfo& systemCodecInfo, AVBufferRef* fr
     }
 
     if (stream == nullptr) {
-        JAMI_ERR("[%p] Can not init, output context has no coding sessions for %s",
+        JAMI_ERR("[%p] Unable to init, output context has no coding sessions for %s",
                  this,
                  systemCodecInfo.name.c_str());
-        throw MediaEncoderException("Cannot allocate stream");
+        throw MediaEncoderException("Unable to allocate stream");
     }
 
     currentStreamIdx_ = stream->index;
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     // Get compatible list of Hardware API
     if (enableAccel_ && mediaType == AVMEDIA_TYPE_VIDEO) {
         auto APIs = video::HardwareAccel::getCompatibleAccel(static_cast<AVCodecID>(
@@ -328,7 +328,7 @@ MediaEncoder::initStream(const SystemCodecInfo& systemCodecInfo, AVBufferRef* fr
         readConfig(encoderCtx);
         encoders_.emplace_back(encoderCtx);
         if (avcodec_open2(encoderCtx, outputCodec_, &options_) < 0)
-            throw MediaEncoderException("Could not open encoder");
+            throw MediaEncoderException("Unable to open encoder");
     }
 
     avcodec_parameters_from_context(stream->codecpar, encoderCtx);
@@ -341,7 +341,7 @@ MediaEncoder::initStream(const SystemCodecInfo& systemCodecInfo, AVBufferRef* fr
         const int width = encoderCtx->width;
         const int height = encoderCtx->height;
         int format = encoderCtx->pix_fmt;
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
         if (accel_) {
             // hardware encoders require a specific pixel format
             auto desc = av_pix_fmt_desc_get(encoderCtx->pix_fmt);
@@ -352,12 +352,12 @@ MediaEncoder::initStream(const SystemCodecInfo& systemCodecInfo, AVBufferRef* fr
         scaledFrameBufferSize_ = videoFrameSize(format, width, height);
         if (scaledFrameBufferSize_ < 0)
             throw MediaEncoderException(
-                ("Could not compute buffer size: " + libav_utils::getError(scaledFrameBufferSize_))
+                ("Unable to compute buffer size: " + libav_utils::getError(scaledFrameBufferSize_))
                     .c_str());
         else if (scaledFrameBufferSize_ <= AV_INPUT_BUFFER_MIN_SIZE)
             throw MediaEncoderException("buffer too small");
 
-        scaledFrameBuffer_.reserve(scaledFrameBufferSize_);
+        scaledFrameBuffer_.resize(scaledFrameBufferSize_);
         scaledFrame_ = std::make_shared<VideoFrame>();
         scaledFrame_->setFromMemory(scaledFrameBuffer_.data(), format, width, height);
     }
@@ -382,7 +382,7 @@ MediaEncoder::openIOContext()
         if (!(outputCtx_->oformat->flags & AVFMT_NOFILE)) {
             fileIO_ = true;
             if ((ret = avio_open(&outputCtx_->pb, filename, AVIO_FLAG_WRITE)) < 0) {
-                throw MediaEncoderException(fmt::format("Could not open IO context for '{}': {}", filename, libav_utils::getError(ret)));
+                throw MediaEncoderException(fmt::format("Unable to open IO context for '{}': {}", filename, libav_utils::getError(ret)));
             }
         }
     }
@@ -394,7 +394,7 @@ MediaEncoder::startIO()
     if (!outputCtx_->pb)
         openIOContext();
     if (avformat_write_header(outputCtx_, options_ ? &options_ : nullptr)) {
-        JAMI_ERR("Could not write header for output file... check codec parameters");
+        JAMI_ERR("Unable to write header for output file... check codec parameters");
         throw MediaEncoderException("Failed to write output file header");
     }
 
@@ -425,14 +425,14 @@ MediaEncoder::encode(const std::shared_ptr<VideoFrame>& input,
     }
 
     std::shared_ptr<VideoFrame> output;
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (getHWFrame(input, output) < 0) {
         JAMI_ERR("Fail to get hardware frame");
         return -1;
     }
 #else
     output = getScaledSWFrame(*input.get());
-#endif // RING_ACCEL
+#endif // ENABLE_HWACCEL
 
     if (!output) {
         JAMI_ERR("Fail to get frame");
@@ -492,7 +492,7 @@ MediaEncoder::encode(AVFrame* frame, int streamIdx)
         }
     }
     int ret = 0;
-    if (streamIdx >= encoders_.size())
+    if (static_cast<size_t>(streamIdx) >= encoders_.size())
         return -1;
     AVCodecContext* encoderCtx = encoders_[streamIdx];
     AVPacket pkt;
@@ -562,7 +562,7 @@ MediaEncoder::flush()
     int ret = 0;
     for (size_t i = 0; i < outputCtx_->nb_streams; ++i) {
         if (encode(nullptr, i) < 0) {
-            JAMI_ERR() << "Could not flush stream #" << i;
+            JAMI_ERR() << "Unable to flush stream #" << i;
             ret |= 1u << i; // provide a way for caller to know which streams failed
         }
     }
@@ -620,7 +620,7 @@ MediaEncoder::prepareEncoderContext(const AVCodec* outputCodec, bool is_video)
         encoderCtx->max_b_frames = 0;
         encoderCtx->pix_fmt = AV_PIX_FMT_YUV420P;
         // Keep YUV format for macOS
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
 #if defined(TARGET_OS_IOS) && TARGET_OS_IOS
         if (accel_)
             encoderCtx->pix_fmt = accel_->getSoftwareFormat();
@@ -636,7 +636,8 @@ MediaEncoder::prepareEncoderContext(const AVCodec* outputCodec, bool is_video)
         // keyframe.
         // encoderCtx->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
     } else {
-        encoderCtx->sample_fmt = AV_SAMPLE_FMT_S16;
+        JAMI_WARNING("Codec format: {} {} {} {}", encoderName, audioOpts_.format, audioOpts_.sampleRate, audioOpts_.nbChannels);
+        encoderCtx->sample_fmt = (AVSampleFormat)audioOpts_.format;
         encoderCtx->sample_rate = std::max(8000, audioOpts_.sampleRate);
         encoderCtx->time_base = AVRational {1, encoderCtx->sample_rate};
         if (audioOpts_.nbChannels > 2 || audioOpts_.nbChannels < 1) {
@@ -659,7 +660,7 @@ MediaEncoder::prepareEncoderContext(const AVCodec* outputCodec, bool is_video)
 void
 MediaEncoder::forcePresetX2645(AVCodecContext* encoderCtx)
 {
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (accel_ && accel_->getName() == "nvenc") {
         if (av_opt_set(encoderCtx, "preset", "fast", AV_OPT_SEARCH_CHILDREN))
             JAMI_WARN("Failed to set preset to 'fast'");
@@ -734,7 +735,7 @@ MediaEncoder::extractProfileLevelID(const std::string& parameters, AVCodecContex
              ctx->level);
 }
 
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
 void
 MediaEncoder::enableAccel(bool enableAccel)
 {
@@ -766,7 +767,7 @@ MediaEncoder::getStream(const std::string& name, int streamIdx) const
     auto enc = encoders_[streamIdx];
     // TODO set firstTimestamp
     auto ms = MediaStream(name, enc);
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (accel_)
         ms.format = accel_->getSoftwareFormat();
 #endif
@@ -777,7 +778,7 @@ AVCodecContext*
 MediaEncoder::initCodec(AVMediaType mediaType, AVCodecID avcodecId, uint64_t br)
 {
     outputCodec_ = nullptr;
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (mediaType == AVMEDIA_TYPE_VIDEO) {
         if (enableAccel_) {
             if (accel_) {
@@ -833,11 +834,13 @@ MediaEncoder::initCodec(AVMediaType mediaType, AVCodecID avcodecId, uint64_t br)
         auto profileLevelId = libav_utils::getDictValue(options_, "parameters");
         extractProfileLevelID(profileLevelId, encoderCtx);
         forcePresetX2645(encoderCtx);
+        encoderCtx->flags2 |= AV_CODEC_FLAG2_LOCAL_HEADER;
         initH264(encoderCtx, br);
     } else if (avcodecId == AV_CODEC_ID_HEVC) {
         encoderCtx->profile = FF_PROFILE_HEVC_MAIN;
         forcePresetX2645(encoderCtx);
         initH265(encoderCtx, br);
+        av_opt_set_int(encoderCtx, "b_ref_mode", 0, AV_OPT_SEARCH_CHILDREN);
     } else if (avcodecId == AV_CODEC_ID_VP8) {
         initVP8(encoderCtx, br);
     } else if (avcodecId == AV_CODEC_ID_MPEG4) {
@@ -852,7 +855,7 @@ MediaEncoder::initCodec(AVMediaType mediaType, AVCodecID avcodecId, uint64_t br)
 int
 MediaEncoder::setBitrate(uint64_t br)
 {
-    std::lock_guard<std::mutex> lk(encMutex_);
+    std::lock_guard lk(encMutex_);
     AVCodecContext* encoderCtx = getCurrentVideoAVCtx();
     if (not encoderCtx)
         return -1; // NOK
@@ -877,7 +880,7 @@ MediaEncoder::setBitrate(uint64_t br)
         // stopEncoder();
         // encoderCtx = initCodec(codecType, codecId, br);
         // if (avcodec_open2(encoderCtx, outputCodec_, &options_) < 0)
-        //     throw MediaEncoderException("Could not open encoder");
+        //     throw MediaEncoderException("Unable to open encoder");
     }
     initAccel(encoderCtx, br);
     return 1; // OK
@@ -886,7 +889,7 @@ MediaEncoder::setBitrate(uint64_t br)
 int
 MediaEncoder::setPacketLoss(uint64_t pl)
 {
-    std::lock_guard<std::mutex> lk(encMutex_);
+    std::lock_guard lk(encMutex_);
     AVCodecContext* encoderCtx = getCurrentAudioAVCtx();
     if (not encoderCtx)
         return -1; // NOK
@@ -1055,7 +1058,7 @@ MediaEncoder::initOpus(AVCodecContext* encoderCtx)
 void
 MediaEncoder::initAccel(AVCodecContext* encoderCtx, uint64_t br)
 {
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (not accel_)
         return;
     if (accel_->getName() == "nvenc"sv) {
@@ -1113,7 +1116,7 @@ MediaEncoder::stopEncoder()
 bool
 MediaEncoder::isDynBitrateSupported(AVCodecID codecid)
 {
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (accel_) {
         return accel_->dynBitrate();
     }
@@ -1136,13 +1139,14 @@ MediaEncoder::isDynPacketLossSupported(AVCodecID codecid)
 void
 MediaEncoder::readConfig(AVCodecContext* encoderCtx)
 {
-    std::string path = fileutils::get_config_dir() + DIR_SEPARATOR_STR + "encoder.json";
+    auto path = fileutils::get_config_dir() / "encoder.json";
     std::string name = encoderCtx->codec->name;
-    if (fileutils::isFile(path)) {
+    std::error_code ec;
+    if (std::filesystem::is_regular_file(path, ec)) {
         JAMI_WARN("encoder.json file found, default settings will be erased");
         try {
             Json::Value root;
-            std::ifstream file = fileutils::ifstream(path);
+            std::ifstream file(path);
             file >> root;
             if (!root.isObject()) {
                 JAMI_ERR() << "Invalid encoder configuration: root is not an object";
@@ -1184,7 +1188,7 @@ MediaEncoder::readConfig(AVCodecContext* encoderCtx)
 std::string
 MediaEncoder::testH265Accel()
 {
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
     if (jami::Manager::instance().videoPreferences.getEncodingAccelerated()) {
         // Get compatible list of Hardware API
         auto APIs = video::HardwareAccel::getCompatibleAccel(AV_CODEC_ID_H265,
@@ -1265,7 +1269,7 @@ MediaEncoder::getHWFrame(const std::shared_ptr<VideoFrame>& input,
         } else {
             output = getScaledSWFrame(*input.get());
         }
-#elif !defined(__APPLE__) && defined(RING_ACCEL)
+#elif !defined(__APPLE__) && defined(ENABLE_HWACCEL)
         // Other Platforms
         auto desc = av_pix_fmt_desc_get(static_cast<AVPixelFormat>(input->format()));
         bool isHardware = desc && (desc->flags & AV_PIX_FMT_FLAG_HWACCEL);
@@ -1295,7 +1299,7 @@ MediaEncoder::getHWFrame(const std::shared_ptr<VideoFrame>& input,
     return 0;
 }
 
-#ifdef RING_ACCEL
+#ifdef ENABLE_HWACCEL
 std::shared_ptr<VideoFrame>
 MediaEncoder::getUnlinkedHWFrame(const VideoFrame& input)
 {
