@@ -1,15 +1,14 @@
 /*
- *  Copyright (C) 2020-2024 Savoir-faire Linux Inc.
- *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
@@ -23,6 +22,7 @@
 #include "../../test_runner.h"
 #include "jami.h"
 #include "account_const.h"
+#include "account_schema.h"
 #include "media_const.h"
 #include "call_const.h"
 #include "common.h"
@@ -76,15 +76,15 @@ private:
     void testDhtPublicInCall();
 
     CPPUNIT_TEST_SUITE(CallTest);
-    //CPPUNIT_TEST(testCall);
-    //CPPUNIT_TEST(testCachedCall);
-    //CPPUNIT_TEST(testStopSearching);
-    //CPPUNIT_TEST(testDeclineMultiDevice);
-    //CPPUNIT_TEST(testTlsInfosPeerCertificate);
-    //CPPUNIT_TEST(testSocketInfos);
-    //CPPUNIT_TEST(testInvalidTurn);
+    CPPUNIT_TEST(testCall);
+    CPPUNIT_TEST(testCachedCall);
+    CPPUNIT_TEST(testStopSearching);
+    CPPUNIT_TEST(testDeclineMultiDevice);
+    CPPUNIT_TEST(testTlsInfosPeerCertificate);
+    CPPUNIT_TEST(testSocketInfos);
+    CPPUNIT_TEST(testInvalidTurn);
     CPPUNIT_TEST(testTransfer);
-    //CPPUNIT_TEST(testDhtPublicInCall);
+    CPPUNIT_TEST(testDhtPublicInCall);
     CPPUNIT_TEST_SUITE_END();
 };
 
@@ -121,7 +121,7 @@ CallTest::testCall()
     auto aliceUri = aliceAccount->getUsername();
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic_bool callReceived {false};
@@ -170,7 +170,7 @@ CallTest::testCachedCall()
     aliceAccount->certStore().pinCertificate(bobAccount->identity().second);
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic_bool callReceived {false}, successfullyConnected {false};
@@ -227,7 +227,7 @@ CallTest::testStopSearching()
     Manager::instance().sendRegister(bobId, false);
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic_bool callStopped {false};
@@ -258,7 +258,7 @@ CallTest::testDeclineMultiDevice()
     auto bobUri = bobAccount->getUsername();
     auto aliceUri = aliceAccount->getUsername();
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
 
     // Add second device for Bob
@@ -273,7 +273,6 @@ CallTest::testDeclineMultiDevice()
     details[ConfProperties::ALIAS] = "BOB2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = bobArchive;
 
     bob2Id = Manager::instance().addAccount(details);
@@ -326,7 +325,7 @@ CallTest::testTlsInfosPeerCertificate()
     auto aliceUri = aliceAccount->getUsername();
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic<int> callStopped {0};
@@ -359,7 +358,7 @@ CallTest::testTlsInfosPeerCertificate()
 
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return !bobCallId.empty(); }));
 
-    Manager::instance().answerCall(bobId, bobCallId);
+    Manager::instance().acceptCall(bobId, bobCallId);
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return aliceCallState == "CURRENT"; }));
 
     auto call = std::dynamic_pointer_cast<SIPCall>(aliceAccount->getCall(callId));
@@ -384,7 +383,7 @@ CallTest::testSocketInfos()
     auto aliceUri = aliceAccount->getUsername();
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic<int> callStopped {0};
@@ -427,7 +426,7 @@ CallTest::testSocketInfos()
 
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return !bobCallId.empty(); }));
 
-    Manager::instance().answerCall(bobId, bobCallId);
+    Manager::instance().acceptCall(bobId, bobCallId);
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return aliceCallState == "CURRENT" && mediaReady; }));
 
     JAMI_INFO("Detail debug");
@@ -456,11 +455,12 @@ CallTest::testInvalidTurn()
     auto aliceUri = aliceAccount->getUsername();
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic_bool callReceived {false};
     std::atomic<int> callStopped {0};
+    bool aliceReady = false;
     // Watch signals
     confHandlers.insert(libjami::exportable_callback<libjami::CallSignal::IncomingCallWithMedia>(
         [&](const std::string&,
@@ -478,11 +478,27 @@ CallTest::testInvalidTurn()
                     cv.notify_one();
             }
         }));
+    confHandlers.insert(
+        libjami::exportable_callback<libjami::ConfigurationSignal::VolatileDetailsChanged>(
+            [&](const std::string& accountId,
+                const std::map<std::string, std::string>& details) {
+                if (accountId != aliceId) {
+                    return;
+                }
+                try {
+                    aliceReady |= accountId == aliceId
+                                && details.at(jami::Conf::CONFIG_ACCOUNT_REGISTRATION_STATUS) == "REGISTERED"
+                                && details.at(libjami::Account::VolatileProperties::DEVICE_ANNOUNCED) == "true";
+                } catch (const std::out_of_range&) {}
+                cv.notify_one();
+            }));
     libjami::registerSignalHandlers(confHandlers);
 
     std::map<std::string, std::string> details;
     details[ConfProperties::TURN::SERVER] = "1.1.1.1";
+    aliceReady = false;
     libjami::setAccountDetails(aliceId, details);
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return aliceReady; }));
 
     JAMI_INFO("Start call between alice and Bob");
     auto call = libjami::placeCallWithMedia(aliceId, bobUri, {});
@@ -509,7 +525,7 @@ CallTest::testTransfer()
     auto aliceUri = aliceAccount->getUsername();
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic_bool bobCallReceived {false};
@@ -578,7 +594,7 @@ CallTest::testDhtPublicInCall()
     auto aliceUri = aliceAccount->getUsername();
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic_bool callReceived {false};
@@ -615,4 +631,4 @@ CallTest::testDhtPublicInCall()
 } // namespace test
 } // namespace jami
 
-RING_TEST_RUNNER(jami::test::CallTest::name())
+CORE_TEST_RUNNER(jami::test::CallTest::name())
