@@ -1,33 +1,22 @@
 /*
- *  Copyright (C) 2017-2023 Savoir-faire Linux Inc.
- *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
-#include <cppunit/TestAssert.h>
-#include <cppunit/TestFixture.h>
-#include <cppunit/extensions/HelperMacros.h>
-
-#include <condition_variable>
-#include <string>
-#include <fstream>
-#include <streambuf>
-
 #include "manager.h"
 #include "jamidht/conversationrepository.h"
-#include "connectivity/connectionmanager.h"
 #include "jamidht/gitserver.h"
 #include "jamidht/jamiaccount.h"
 #include "../../test_runner.h"
@@ -38,6 +27,17 @@
 #include "common.h"
 
 #include <git2.h>
+
+#include <dhtnet/connectionmanager.h>
+
+#include <cppunit/TestAssert.h>
+#include <cppunit/TestFixture.h>
+#include <cppunit/extensions/HelperMacros.h>
+
+#include <condition_variable>
+#include <string>
+#include <fstream>
+#include <streambuf>
 #include <filesystem>
 
 using namespace std::string_literals;
@@ -64,6 +64,9 @@ public:
 
     std::string aliceId;
     std::string bobId;
+    std::mutex mtx;
+    std::unique_lock<std::mutex> lk {mtx};
+    std::condition_variable cv;
 
 private:
     void testCreateRepository();
@@ -74,7 +77,9 @@ private:
     void testDiff();
 
     void testMergeProfileWithConflict();
-
+    void testValidSignatureForCommit();
+    void testInvalidSignatureForCommit();
+    void testMalformedCommit();
     std::string addCommit(git_repository* repo,
                           const std::shared_ptr<JamiAccount> account,
                           const std::string& branch,
@@ -83,16 +88,17 @@ private:
     bool merge_in_main(const std::shared_ptr<JamiAccount> account,
                        git_repository* repo,
                        const std::string& commit_ref);
-
     CPPUNIT_TEST_SUITE(ConversationRepositoryTest);
-    CPPUNIT_TEST(testCreateRepository);
-    CPPUNIT_TEST(testAddSomeMessages);
-    CPPUNIT_TEST(testLogMessages);
-    CPPUNIT_TEST(testMerge);
-    CPPUNIT_TEST(testFFMerge);
-    CPPUNIT_TEST(testDiff);
-    CPPUNIT_TEST(testMergeProfileWithConflict);
-
+    CPPUNIT_TEST(testCreateRepository);          // Passes
+    CPPUNIT_TEST(testAddSomeMessages);           // Passes
+    CPPUNIT_TEST(testLogMessages);               // Passes
+    CPPUNIT_TEST(testMerge);                     // Passes
+    CPPUNIT_TEST(testFFMerge);                   // Passes
+    CPPUNIT_TEST(testDiff);                      // Passes
+    CPPUNIT_TEST(testMergeProfileWithConflict);  // Passes
+    CPPUNIT_TEST(testValidSignatureForCommit);   // Passes
+    CPPUNIT_TEST(testInvalidSignatureForCommit); // Passes
+    CPPUNIT_TEST(testMalformedCommit);           // Passes
     CPPUNIT_TEST_SUITE_END();
 };
 
@@ -124,9 +130,9 @@ ConversationRepositoryTest::testCreateRepository()
 
     // Assert that repository exists
     CPPUNIT_ASSERT(repository != nullptr);
-    auto repoPath = fileutils::get_data_dir() + DIR_SEPARATOR_STR + aliceAccount->getAccountID()
-                    + DIR_SEPARATOR_STR + "conversations" + DIR_SEPARATOR_STR + repository->id();
-    CPPUNIT_ASSERT(fileutils::isDirectory(repoPath));
+    auto repoPath = fileutils::get_data_dir() / aliceAccount->getAccountID() / "conversations"
+                    / repository->id();
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
 
     // Assert that first commit is signed by alice
     git_repository* repo;
@@ -145,12 +151,11 @@ ConversationRepositoryTest::testCreateRepository()
     CPPUNIT_ASSERT(aliceAccount->identity().second->getPublicKey().checkSignature(data, pk));
 
     // 2. Check created files
-    auto CRLsPath = repoPath + DIR_SEPARATOR_STR + "CRLs" + DIR_SEPARATOR_STR
-                    + aliceDeviceId.toString();
-    CPPUNIT_ASSERT(fileutils::isDirectory(repoPath));
+    auto CRLsPath = repoPath / "CRLs" / aliceDeviceId.toString();
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
 
-    auto adminCrt = repoPath + DIR_SEPARATOR_STR + "admins" + DIR_SEPARATOR_STR + uri + ".crt";
-    CPPUNIT_ASSERT(fileutils::isFile(adminCrt));
+    auto adminCrt = repoPath / "admins" / (uri + ".crt");
+    CPPUNIT_ASSERT(std::filesystem::is_regular_file(adminCrt));
 
     auto crt = std::ifstream(adminCrt);
     std::string adminCrtStr((std::istreambuf_iterator<char>(crt)), std::istreambuf_iterator<char>());
@@ -161,9 +166,8 @@ ConversationRepositoryTest::testCreateRepository()
 
     CPPUNIT_ASSERT(adminCrtStr == parentCert);
 
-    auto deviceCrt = repoPath + DIR_SEPARATOR_STR + "devices" + DIR_SEPARATOR_STR
-                     + aliceDeviceId.toString() + ".crt";
-    CPPUNIT_ASSERT(fileutils::isFile(deviceCrt));
+    auto deviceCrt = repoPath / "devices" / (aliceDeviceId.toString() + ".crt");
+    CPPUNIT_ASSERT(std::filesystem::is_regular_file(deviceCrt));
 
     crt = std::ifstream(deviceCrt);
     std::string deviceCrtStr((std::istreambuf_iterator<char>(crt)),
@@ -262,13 +266,13 @@ ConversationRepositoryTest::addCommit(git_repository* repo,
     // Retrieve current HEAD
     git_oid commit_id;
     if (git_reference_name_to_id(&commit_id, repo, "HEAD") < 0) {
-        JAMI_ERR("Cannot get reference for HEAD");
+        JAMI_ERR("Unable to get reference for HEAD");
         return {};
     }
 
     git_commit* head_ptr = nullptr;
     if (git_commit_lookup(&head_ptr, repo, &commit_id) < 0) {
-        JAMI_ERR("Could not look up HEAD commit");
+        JAMI_ERR("Unable to look up HEAD commit");
         return {};
     }
     GitCommit head_commit {head_ptr, git_commit_free};
@@ -276,7 +280,7 @@ ConversationRepositoryTest::addCommit(git_repository* repo,
     // Retrieve current index
     git_index* index_ptr = nullptr;
     if (git_repository_index(&index_ptr, repo) < 0) {
-        JAMI_ERR("Could not open repository index");
+        JAMI_ERR("Unable to open repository index");
         return {};
     }
     GitIndex index {index_ptr, git_index_free};
@@ -289,13 +293,18 @@ ConversationRepositoryTest::addCommit(git_repository* repo,
 
     git_tree* tree_ptr = nullptr;
     if (git_tree_lookup(&tree_ptr, repo, &tree_id) < 0) {
-        JAMI_ERR("Could not look up initial tree");
+        JAMI_ERR("Unable to look up initial tree");
         return {};
     }
     GitTree tree = {tree_ptr, git_tree_free};
 
     git_buf to_sign = {};
+#if LIBGIT2_VER_MAJOR == 1 && LIBGIT2_VER_MINOR == 8 \
+    && (LIBGIT2_VER_REVISION == 0 || LIBGIT2_VER_REVISION == 1 || LIBGIT2_VER_REVISION == 3)
+    git_commit* const head_ref[1] = {head_commit.get()};
+#else
     const git_commit* head_ref[1] = {head_commit.get()};
+#endif
     if (git_commit_create_buffer(&to_sign,
                                  repo,
                                  sig.get(),
@@ -306,7 +315,7 @@ ConversationRepositoryTest::addCommit(git_repository* repo,
                                  1,
                                  &head_ref[0])
         < 0) {
-        JAMI_ERR("Could not create commit buffer");
+        JAMI_ERR("Unable to create commit buffer");
         return {};
     }
 
@@ -320,7 +329,7 @@ ConversationRepositoryTest::addCommit(git_repository* repo,
                                          signed_str.c_str(),
                                          "signature")
         < 0) {
-        JAMI_ERR("Could not sign commit");
+        JAMI_ERR("Unable to sign commit");
         return {};
     }
 
@@ -332,7 +341,7 @@ ConversationRepositoryTest::addCommit(git_repository* repo,
         std::string branch_name = "refs/heads/" + branch;
         if (git_reference_create(&ref_ptr, repo, branch_name.c_str(), &commit_id, true, nullptr)
             < 0) {
-            JAMI_WARN("Could not move commit to main");
+            JAMI_WARN("Unable to move commit to main");
         }
         git_reference_free(ref_ptr);
     }
@@ -361,9 +370,9 @@ ConversationRepositoryTest::testMerge()
 
     // Assert that repository exists
     CPPUNIT_ASSERT(repository != nullptr);
-    auto repoPath = fileutils::get_data_dir() + DIR_SEPARATOR_STR + aliceAccount->getAccountID()
-                    + DIR_SEPARATOR_STR + "conversations" + DIR_SEPARATOR_STR + repository->id();
-    CPPUNIT_ASSERT(fileutils::isDirectory(repoPath));
+    auto repoPath = fileutils::get_data_dir() / aliceAccount->getAccountID() / "conversations"
+                    / repository->id();
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
 
     // Assert that first commit is signed by alice
     git_repository* repo;
@@ -396,9 +405,9 @@ ConversationRepositoryTest::testFFMerge()
 
     // Assert that repository exists
     CPPUNIT_ASSERT(repository != nullptr);
-    auto repoPath = fileutils::get_data_dir() + DIR_SEPARATOR_STR + aliceAccount->getAccountID()
-                    + DIR_SEPARATOR_STR + "conversations" + DIR_SEPARATOR_STR + repository->id();
-    CPPUNIT_ASSERT(fileutils::isDirectory(repoPath));
+    auto repoPath = fileutils::get_data_dir() / aliceAccount->getAccountID() / "conversations"
+                    / repository->id();
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
 
     // Assert that first commit is signed by alice
     git_repository* repo;
@@ -452,15 +461,15 @@ ConversationRepositoryTest::testMergeProfileWithConflict()
 
     // Assert that repository exists
     CPPUNIT_ASSERT(repository != nullptr);
-    auto repoPath = fileutils::get_data_dir() + DIR_SEPARATOR_STR + aliceAccount->getAccountID()
-                    + DIR_SEPARATOR_STR + "conversations" + DIR_SEPARATOR_STR + repository->id();
-    CPPUNIT_ASSERT(fileutils::isDirectory(repoPath));
+    auto repoPath = fileutils::get_data_dir() / aliceAccount->getAccountID() / "conversations"
+                    / repository->id();
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
 
     // Assert that first commit is signed by alice
     git_repository* repo;
     CPPUNIT_ASSERT(git_repository_open(&repo, repoPath.c_str()) == 0);
 
-    auto profile = std::ofstream(repoPath + DIR_SEPARATOR_STR + "profile.vcf");
+    auto profile = std::ofstream(repoPath / "profile.vcf");
     if (profile.is_open()) {
         profile << "TITLE: SWARM\n";
         profile << "SUBTITLE: Some description\n";
@@ -469,7 +478,7 @@ ConversationRepositoryTest::testMergeProfileWithConflict()
     }
     addAll(repo);
     auto id1 = addCommit(repo, aliceAccount, "main", "add profile");
-    profile = std::ofstream(repoPath + DIR_SEPARATOR_STR + "profile.vcf");
+    profile = std::ofstream(repoPath / "profile.vcf");
     if (profile.is_open()) {
         profile << "TITLE: SWARM\n";
         profile << "SUBTITLE: New description\n";
@@ -488,7 +497,7 @@ ConversationRepositoryTest::testMergeProfileWithConflict()
     git_reference_free(ref);
     git_repository_set_head(repo, "refs/heads/to_merge");
 
-    profile = std::ofstream(repoPath + DIR_SEPARATOR_STR + "profile.vcf");
+    profile = std::ofstream(repoPath / "profile.vcf");
     if (profile.is_open()) {
         profile << "TITLE: SWARM\n";
         profile << "SUBTITLE: Another description\n";
@@ -503,7 +512,233 @@ ConversationRepositoryTest::testMergeProfileWithConflict()
     CPPUNIT_ASSERT(repository->log().size() == 5 /* Initial, add, modify 1, modify 2, merge */);
 }
 
+void
+ConversationRepositoryTest::testValidSignatureForCommit()
+{
+    // Get Alice's account
+    auto aliceAccount = Manager::instance().getAccount<JamiAccount>(aliceId);
+    // Create a repository associated with Alice's account
+    auto repository = ConversationRepository::createConversation(aliceAccount);
+    // Assert the repository exists
+    CPPUNIT_ASSERT(repository != nullptr);
+    // Get the path to the conversation repository
+    auto repoPath = fileutils::get_data_dir() / aliceAccount->getAccountID() / "conversations"
+                    / repository->id();
+    // Assert that the repository path is a directory
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
+
+    // Assert that the git repo was opened successfully
+    git_repository* repo;
+    CPPUNIT_ASSERT(git_repository_open(&repo, repoPath.c_str()) == 0);
+
+    // Get the initial commit
+    git_oid commit_id;
+    CPPUNIT_ASSERT(git_reference_name_to_id(&commit_id, repo, "HEAD") == 0);
+
+    // Get the commit object
+    git_commit* commit_ptr = nullptr;
+    CPPUNIT_ASSERT(git_commit_lookup(&commit_ptr, repo, &commit_id) == 0);
+
+    // Extract the author email from the commit
+    const git_signature* author = git_commit_author(commit_ptr);
+    std::string userDevice = author->email;
+
+    // Extract the signature and signature data of commit_id
+    git_buf extracted_sig = {}, extracted_sig_data = {};
+    // Extract the signature block and signature content from the commit
+    git_commit_extract_signature(&extracted_sig, &extracted_sig_data, repo, &commit_id, "signature");
+    // Verify the signature of the commit
+    CPPUNIT_ASSERT(repository->isValidUserAtCommit(userDevice,
+                                                   git_oid_tostr_s(&commit_id),
+                                                   extracted_sig,
+                                                   extracted_sig_data));
+
+    // Free git objects
+    git_commit_free(commit_ptr);
+    git_repository_free(repo);
+}
+
+void
+ConversationRepositoryTest::testInvalidSignatureForCommit()
+{
+    // Get Alice and Bob's accounts
+    auto aliceAccount = Manager::instance().getAccount<JamiAccount>(aliceId);
+    auto bobAccount = Manager::instance().getAccount<JamiAccount>(bobId);
+    // Create a repository associated with Alice's account
+    auto repository = ConversationRepository::createConversation(aliceAccount);
+    // Assert that the repository exists
+    CPPUNIT_ASSERT(repository != nullptr);
+    // Get the path to the conversation repository
+    auto repoPath = fileutils::get_data_dir() / aliceAccount->getAccountID() / "conversations"
+                    / repository->id();
+    // Assert that the repository path is a directory
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
+
+    // Assert that the git repo was opened successfully
+    git_repository* repo;
+    CPPUNIT_ASSERT(git_repository_open(&repo, repoPath.c_str()) == 0);
+
+    // Make a malicious commit that disguises itself as being sent from Alice
+    // Retrieve current index
+    git_index* index_ptr = nullptr;
+    CPPUNIT_ASSERT(git_repository_index(&index_ptr, repo) == 0);
+    GitIndex index {index_ptr, git_index_free};
+
+    // Write the index to a new tree
+    git_oid tree_id;
+    // Assert that the tree was written successfully
+    CPPUNIT_ASSERT(git_index_write_tree(&tree_id, index.get()) == 0);
+
+    // Lookup the newly created tree
+    git_tree* tree_ptr = nullptr;
+    CPPUNIT_ASSERT(git_tree_lookup(&tree_ptr, repo, &tree_id) == 0);
+    GitTree tree = {tree_ptr, git_tree_free};
+
+    // Create a commit object
+    git_oid commit_id;
+    CPPUNIT_ASSERT(git_reference_name_to_id(&commit_id, repo, "HEAD") == 0);
+
+    // Lookup the HEAD commit
+    git_commit* head_ptr = nullptr;
+    CPPUNIT_ASSERT(git_commit_lookup(&head_ptr, repo, &commit_id) == 0);
+    GitCommit head_commit {head_ptr, git_commit_free};
+
+    // The last argument of git_commit_create_buffer is of type
+    // 'const git_commit **' in all versions of libgit2 except 1.8.0,
+    // 1.8.1 and 1.8.3, in which it is of type 'git_commit *const *'.
+#if LIBGIT2_VER_MAJOR == 1 && LIBGIT2_VER_MINOR == 8 \
+    && (LIBGIT2_VER_REVISION == 0 || LIBGIT2_VER_REVISION == 1 || LIBGIT2_VER_REVISION == 3)
+    git_commit* const head_ref[1] = {head_commit.get()};
+#else
+    const git_commit* head_ref[1] = {head_commit.get()};
+#endif
+
+    // Create the signature for the commit with the current time and using the
+    // Alice's device ID (i.e. her public key for her device)
+    git_signature* sig = nullptr;
+    CPPUNIT_ASSERT(git_signature_now(&sig,
+                                     aliceAccount->getDisplayName().c_str(),
+                                     std::string(aliceAccount->currentDeviceId()).c_str())
+                   == 0);
+
+    // Create a signed commit object and place it in a buffer
+    git_buf to_sign = {};
+    CPPUNIT_ASSERT(
+        git_commit_create_buffer(&to_sign,
+                                 repo,
+                                 sig,
+                                 sig,
+                                 nullptr,
+                                 "{\"body\":\"malicious intent\",\"type\":\"text/plain\"}",
+                                 tree.get(),
+                                 1,
+                                 &head_ref[0])
+        == 0);
+    git_signature_free(sig);
+
+    // git commit -S
+    auto to_sign_vec = std::vector<uint8_t>(to_sign.ptr, to_sign.ptr + to_sign.size);
+    // Sign the commit using Bob's private key
+    std::vector<unsigned char> signed_buf = bobAccount->identity().first->sign(to_sign_vec);
+    std::string signed_str = base64::encode(signed_buf);
+    CPPUNIT_ASSERT(git_commit_create_with_signature(&commit_id,
+                                                    repo,
+                                                    to_sign.ptr,
+                                                    signed_str.c_str(),
+                                                    "signature")
+                   == 0);
+
+    // Dispose the signature buffer
+    git_buf_dispose(&to_sign);
+
+    // Move commit to main branch
+    git_reference* ref_ptr = nullptr;
+    CPPUNIT_ASSERT(
+        (git_reference_create(&ref_ptr, repo, "refs/heads/main", &commit_id, true, nullptr) == 0));
+
+    CPPUNIT_ASSERT(repo);
+
+    // Lookup the commit
+    git_commit* commit_ptr = nullptr;
+    CPPUNIT_ASSERT(git_commit_lookup(&commit_ptr, repo, &commit_id) == 0);
+
+    // Get the author and user device, and assert that the user is not valid at the commit
+    const git_signature* author = git_commit_author(commit_ptr);
+    std::string userDevice = author->email;
+
+    // Extract the signature and signature data of commit_id
+    git_buf extracted_sig = {}, extracted_sig_data = {};
+    // Extract the signature block and signature content from the commit
+    git_commit_extract_signature(&extracted_sig, &extracted_sig_data, repo, &commit_id, "signature");
+    CPPUNIT_ASSERT(!repository->isValidUserAtCommit(userDevice,
+                                                    git_oid_tostr_s(&commit_id),
+                                                    extracted_sig,
+                                                    extracted_sig_data));
+
+    // Free git objects
+    git_reference_free(ref_ptr);
+    git_repository_free(repo);
+    git_buf_dispose(&extracted_sig);
+    git_buf_dispose(&extracted_sig_data);
+}
+
+void
+ConversationRepositoryTest::testMalformedCommit()
+{
+    // Register signals
+    std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
+
+    // Variable to be set to true on detection of malformed commit
+    bool isMalformed = false;
+
+    // Signals for malformed commits in validCommits function
+    confHandlers.insert(
+        libjami::exportable_callback<libjami::ConversationSignal::OnConversationError>(
+            [&](const std::string& accountId,
+                const std::string& conversationId,
+                int code,
+                auto malformedSpec) {
+                if (code == EVALIDFETCH) {
+                    isMalformed = true;
+                }
+                cv.notify_one();
+            }));
+    libjami::registerSignalHandlers(confHandlers);
+
+    // Get Alice's account
+    auto aliceAccount = Manager::instance().getAccount<JamiAccount>(aliceId);
+    // Create a repository associated with Alice's account
+    auto repository = ConversationRepository::createConversation(aliceAccount);
+    // Assert that repository exists
+    CPPUNIT_ASSERT(repository != nullptr);
+    // Get the path to the conversation repository
+    auto repoPath = fileutils::get_data_dir() / aliceAccount->getAccountID() / "conversations"
+                    / repository->id();
+    // Assert that the repository path is a directory
+    CPPUNIT_ASSERT(std::filesystem::is_directory(repoPath));
+
+    // Assert that the git repo was opened successfully
+    git_repository* repo;
+    CPPUNIT_ASSERT(git_repository_open(&repo, repoPath.c_str()) == 0);
+
+    // Create a malformed join commit
+    auto malformedJoinID = addCommit(repo,
+                                     aliceAccount,
+                                     "main",
+                                     "{\"action\":\"join\",\"type\":\"member\",\"uri\":"
+                                     "\"joinGarbage\"}");
+
+    // Log the repository
+    auto conversationCommits = repository->log();
+    // Call the validation for all the commits
+    bool allCommitsValidated = repository->validCommits(conversationCommits);
+    // Check that the appropriate signal was called
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return isMalformed; }));
+    // Check that the return value was false (i.e. malformed commit detected)
+    CPPUNIT_ASSERT(!allCommitsValidated);
+}
+
 } // namespace test
 } // namespace jami
 
-RING_TEST_RUNNER(jami::test::ConversationRepositoryTest::name())
+CORE_TEST_RUNNER(jami::test::ConversationRepositoryTest::name())
