@@ -1,15 +1,14 @@
 /*
- *  Copyright (C) 2022-2024 Savoir-faire Linux Inc.
- *  Author: Aline Gondim Santos <aline.gondimsantos@savoirfairelinux.com>
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
@@ -180,7 +179,7 @@ PluginsTest::onIncomingCallWithMedia(const std::string& accountId,
         return;
     }
 
-    std::unique_lock<std::mutex> lock {callData.mtx_};
+    std::unique_lock lock {callData.mtx_};
     callData.callId_ = callId;
     callData.signals_.emplace_back(CallData::Signal(libjami::CallSignal::IncomingCallWithMedia::name));
 
@@ -202,7 +201,7 @@ PluginsTest::onCallStateChange(const std::string& accountId,
     CPPUNIT_ASSERT(accountId == callData.accountId_);
 
     {
-        std::unique_lock<std::mutex> lock {callData.mtx_};
+        std::unique_lock lock {callData.mtx_};
         callData.signals_.emplace_back(
             CallData::Signal(libjami::CallSignal::StateChange::name, state));
     }
@@ -547,7 +546,7 @@ PluginsTest::waitForSignal(CallData& callData,
                             const std::string& expectedEvent)
 {
     const std::chrono::seconds TIME_OUT {30};
-    std::unique_lock<std::mutex> lock {callData.mtx_};
+    std::unique_lock lock {callData.mtx_};
 
     // Combined signal + event (if any).
     std::string sigEvent(expectedSignal);
@@ -691,7 +690,7 @@ PluginsTest::testMessage()
     // end call
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     auto messageBobReceived = 0, messageAliceReceived = 0;
@@ -769,4 +768,4 @@ PluginsTest::testMessage()
 } // namespace test
 } // namespace jami
 
-RING_TEST_RUNNER(jami::test::PluginsTest::name())
+CORE_TEST_RUNNER(jami::test::PluginsTest::name())
