@@ -1,21 +1,18 @@
 /*
- *  Copyright (C) 2024 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  Author: Fadi Shehadeh <fadi.shehadeh@savoirfairelinux.com>
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <cppunit/TestAssert.h>
@@ -162,8 +159,7 @@ SwarmConversationTest::testSendMessage()
     }
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
-    std::condition_variable cv;
+    std::unique_lock lk {mtx};
 
     connectSignals();
 
@@ -182,7 +178,7 @@ SwarmConversationTest::testSendMessage()
         }));
 
         libjami::acceptConversationRequest(it->first, convId);
-        std::this_thread::sleep_for(std::chrono::seconds(10));
+        std::this_thread::sleep_for(std::chrono::seconds(5));
     }
 
     std::cout << "waiting for conversation ready" << std::endl;
@@ -219,4 +215,4 @@ SwarmConversationTest::testSendMessage()
 } // namespace test
 } // namespace jami
 
-RING_TEST_RUNNER(jami::test::SwarmConversationTest::name())
+CORE_TEST_RUNNER(jami::test::SwarmConversationTest::name())
