@@ -1,15 +1,14 @@
 /*
- *  Copyright (C) 2017-2024 Savoir-faire Linux Inc.
- *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
@@ -57,6 +56,7 @@ struct UserData {
     std::vector<libjami::SwarmMessage> messages;
     std::vector<libjami::SwarmMessage> messagesLoaded;
     std::vector<libjami::SwarmMessage> messagesUpdated;
+    std::map<std::string, int> members;
 };
 
 class SyncHistoryTest : public CppUnit::TestFixture
@@ -210,6 +210,17 @@ SyncHistoryTest::connectSignals()
                 alice2Data.profilePath = path;
             cv.notify_one();
         }));
+    confHandlers.insert(libjami::exportable_callback<libjami::ConversationSignal::ConversationMemberEvent>(
+        [&](const std::string& accountId, const std::string& conversationId, const auto& member, auto status) {
+            if (accountId == aliceId) {
+                aliceData.members[member] = status;
+            } else if (accountId == bobId) {
+                bobData.members[member] = status;
+            } else if (accountId == alice2Id) {
+                alice2Data.members[member] = status;
+            }
+            cv.notify_one();
+        }));
     confHandlers.insert(
         libjami::exportable_callback<libjami::ConfigurationSignal::AccountMessageStatusChanged>(
             [&](const std::string& accountId,
@@ -283,6 +294,8 @@ SyncHistoryTest::connectSignals()
                 aliceData.messages.emplace_back(message);
             } else if (accountId == bobId) {
                 bobData.messages.emplace_back(message);
+            } else if (accountId == alice2Id) {
+                alice2Data.messages.emplace_back(message);
             }
             cv.notify_one();
         }));
@@ -354,7 +367,6 @@ SyncHistoryTest::testCreateConversationThenSync()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
 
     alice2Id = Manager::instance().addAccount(details);
@@ -376,7 +388,6 @@ SyncHistoryTest::testCreateConversationWithOnlineDevice()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     auto convId = libjami::startConversation(aliceId);
     alice2Id = Manager::instance().addAccount(details);
@@ -408,7 +419,6 @@ SyncHistoryTest::testCreateConversationWithMessagesThenAddDevice()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return !alice2Data.conversationId.empty(); }));
@@ -456,12 +466,11 @@ SyncHistoryTest::testCreateMultipleConversationThenAddDevice()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
 
     std::mutex mtx;
-    std::unique_lock<std::mutex> lk {mtx};
+    std::unique_lock lk {mtx};
     std::condition_variable cv;
     std::map<std::string, std::shared_ptr<libjami::CallbackWrapperBase>> confHandlers;
     std::atomic_int conversationReady = 0;
@@ -504,7 +513,6 @@ SyncHistoryTest::testReceivesInviteThenAddDevice()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
 
@@ -526,7 +534,6 @@ SyncHistoryTest::testRemoveConversationOnAllDevices()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     auto convId = libjami::startConversation(aliceId);
 
@@ -567,12 +574,11 @@ SyncHistoryTest::testSyncCreateAccountExportDeleteReimportOldBackup()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return alice2Data.deviceAnnounced; }));
 
-    // This will trigger a conversation request. Cause alice2 can't know first conversation
+    // This will trigger a conversation request. Cause alice2 is unable to know of first conversation
     libjami::sendMessage(bobId, convId, std::string("hi"), "");
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return alice2Data.requestReceived; }));
 
@@ -620,7 +626,6 @@ SyncHistoryTest::testSyncCreateAccountExportDeleteReimportWithConvId()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
     // Should retrieve conversation, no need for action as the convInfos is in the archive
@@ -658,7 +663,6 @@ SyncHistoryTest::testSyncCreateAccountExportDeleteReimportWithConvReq()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&] { return alice2Data.deviceAnnounced; }));
@@ -687,7 +691,6 @@ SyncHistoryTest::testSyncOneToOne()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
 
     alice2Id = Manager::instance().addAccount(details);
@@ -721,7 +724,6 @@ SyncHistoryTest::testConversationRequestRemoved()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
 
@@ -788,7 +790,6 @@ END:VCARD";
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     bobData.profilePath = {};
     alice2Data.profilePath = {};
@@ -836,7 +837,6 @@ SyncHistoryTest::testLastInteractionAfterClone()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
 
@@ -871,37 +871,58 @@ SyncHistoryTest::testLastInteractionAfterSomeMessages()
     details[ConfProperties::ALIAS] = "ALICE2";
     details[ConfProperties::UPNP_ENABLED] = "true";
     details[ConfProperties::ARCHIVE_PASSWORD] = "";
-    details[ConfProperties::ARCHIVE_PIN] = "";
     details[ConfProperties::ARCHIVE_PATH] = aliceArchive;
     alice2Id = Manager::instance().addAccount(details);
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() {return alice2Data.deviceAnnounced; }));
+    auto getMessageFromBody = [](const auto& data, const auto& body) -> std::string {
+        auto it = std::find_if(data.messages.begin(), data.messages.end(), [&](auto& msg) {
+            return msg.body.find("body") != msg.body.end() && msg.body.at("body") == body; });
+        if (it != data.messages.end()) {
+            return it->id;
+        }
+        return {};
+    };
+    auto getMessage = [](const auto& data, const auto& mid) -> bool {
+        return std::find_if(data.messages.begin(), data.messages.end(), [&](auto& msg) { return msg.id == mid; }) != data.messages.end();
+    };
 
     aliceAccount->addContact(bobUri);
     aliceAccount->sendTrustRequest(bobUri, {});
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return bobData.requestReceived && !alice2Data.conversationId.empty(); }));
 
-    auto aliceMsgSize = aliceData.messages.size();
     CPPUNIT_ASSERT(bobAccount->acceptTrustRequest(aliceUri));
     CPPUNIT_ASSERT(
-        cv.wait_for(lk, 30s, [&]() { return aliceMsgSize + 1 == aliceData.messages.size(); }));
+        cv.wait_for(lk, 30s, [&]() { return aliceData.members[bobUri] == 1 && alice2Data.members[bobUri] == 1; }));
 
     // Start conversation
+    bobData.messages.clear();
     libjami::sendMessage(bobId, aliceData.conversationId, std::string("Message 1"), "");
-    CPPUNIT_ASSERT(cv.wait_for(lk, 10s, [&] { return aliceMsgSize + 2 == aliceData.messages.size(); }));
-    libjami::sendMessage(bobId, aliceData.conversationId, std::string("Message 2"), "");
-    CPPUNIT_ASSERT(cv.wait_for(lk, 10s, [&] { return aliceMsgSize + 3 == aliceData.messages.size(); }));
-    libjami::sendMessage(bobId, aliceData.conversationId, std::string("Message 3"), "");
-    CPPUNIT_ASSERT(cv.wait_for(lk, 10s, [&] { return aliceMsgSize + 4 == aliceData.messages.size(); }));
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return !getMessageFromBody(bobData, "Message 1").empty(); }));
+    auto msgId = getMessageFromBody(bobData, "Message 1");
+    CPPUNIT_ASSERT(cv.wait_for(lk, 20s, [&] { return getMessage(aliceData, msgId) && getMessage(alice2Data, msgId); }));
 
-    auto displayedId = aliceData.messages.rbegin()->id;
-    libjami::setMessageDisplayed(aliceId, "swarm:" + aliceData.conversationId, displayedId, 3);
-    CPPUNIT_ASSERT(cv.wait_for(lk, 20s, [&] { return aliceData.sent && alice2Data.sent; }));
+    bobData.messages.clear();
+    libjami::sendMessage(bobId, aliceData.conversationId, std::string("Message 2"), "");
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return !getMessageFromBody(bobData, "Message 2").empty(); }));
+    msgId = getMessageFromBody(bobData, "Message 2");
+    CPPUNIT_ASSERT(cv.wait_for(lk, 10s, [&] { return getMessage(aliceData, msgId) && getMessage(alice2Data, msgId); }));
+
+    bobData.messages.clear();
+    libjami::sendMessage(bobId, aliceData.conversationId, std::string("Message 3"), "");
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return !getMessageFromBody(bobData, "Message 3").empty(); }));
+    msgId = getMessageFromBody(bobData, "Message 3");
+    CPPUNIT_ASSERT(cv.wait_for(lk, 10s, [&] { return getMessage(aliceData, msgId) && getMessage(alice2Data, msgId); }));
+
+    libjami::setMessageDisplayed(aliceId, "swarm:" + aliceData.conversationId, msgId, 3);
+    CPPUNIT_ASSERT(cv.wait_for(lk, 20s, [&] {
+        return aliceData.sent && alice2Data.sent; }));
 
     auto membersInfos = libjami::getConversationMembers(alice2Id, alice2Data.conversationId);
     CPPUNIT_ASSERT(std::find_if(membersInfos.begin(),
                                 membersInfos.end(),
                                 [&](auto infos) {
                                     return infos["uri"] == aliceUri
-                                           && infos["lastDisplayed"] == displayedId;
+                                           && infos["lastDisplayed"] == msgId;
                                 })
                    != membersInfos.end());
 }
@@ -909,4 +930,4 @@ SyncHistoryTest::testLastInteractionAfterSomeMessages()
 } // namespace test
 } // namespace jami
 
-RING_TEST_RUNNER(jami::test::SyncHistoryTest::name())
+CORE_TEST_RUNNER(jami::test::SyncHistoryTest::name())
