@@ -1,22 +1,21 @@
 #! /usr/bin/env python3
 #
-#  Copyright (C) 2004-2024 Savoir-faire Linux Inc. Inc
+#  Copyright (C) 2004-2025 Savoir-faire Linux Inc. Inc
 #
 # Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
 #
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+# along with this program. If not, see <https://www.gnu.org/licenses/>.
 #
 
 """libjami controlling class through DBUS"""
@@ -298,10 +297,10 @@ class libjamiCtrl(Thread):
     def onConferenceCreated_cb(self):
         pass
 
-    def onConferenceCreated_callback(self, confId):
+    def onConferenceCreated_callback(self, convId, confId):
         pass
 
-    def onConferenceCreated(self, confId):
+    def onConferenceCreated(self, convId, confId):
         self.currentConfId = confId
         self.onConferenceCreated_cb()
         self.onConferenceCreated_callback(confId)
@@ -540,12 +539,12 @@ class libjamiCtrl(Thread):
         return [str(x) for x in self.callmanager.getConferenceList()]
 
     def getCallDetails(self, callid):
-        """Return informations on this call if exists"""
+        """Return information on this call if exists"""
 
         return self.callmanager.getCallDetails(callid)
 
     def getConferenceDetails(self, confid):
-        """Return informations on this conference if exists"""
+        """Return information on this conference if exists"""
 
         return self.callmanager.getConferenceDetails(confid)
 
@@ -572,7 +571,7 @@ class libjamiCtrl(Thread):
             self.setFirstRegisteredAccount()
 
         if self.account != "IP2IP" and not self.isAccountRegistered():
-            raise libjamiCtrlAccountError("Can't place a call without a registered account")
+            raise libjamiCtrlAccountError("Unable to place a call without a registered account")
 
         # Send the request to the CallManager
         callid = self.callmanager.placeCall(self.account, dest)
@@ -622,7 +621,7 @@ class libjamiCtrl(Thread):
             self.setFirstRegisteredAccount()
 
         if not self.isAccountRegistered():
-            raise libjamiCtrlAccountError("Can't accept a call without a registered account")
+            raise libjamiCtrlAccountError("Unable to accept a call without a registered account")
 
         if callid is None or callid == "":
             raise libjamiCtrlError("Invalid callID")
