@@ -9,14 +9,15 @@ and package the project for Windows.
 usage: build.py [-q] [-h] [-a ARCH] [-c CONFIG] [-t] [-i] [-v] {pack} ...
 
 optional arguments:
-  -q, --qt PATH         Sets the Qt installation path
-  -a ARCH, --arch ARCH  Sets the build architecture
+  -q, --qt PATH             Sets the Qt installation path
+  -a ARCH, --arch ARCH      Sets the build architecture
   -c CONFIG, --config CONFIG
-                        Sets the build configuration type
-  -t, --tests           Build and run tests
-  -i, --init            Initialize submodules
-  -v, --version         Show the version number and exit
-  -s, --skip-build      Only do packaging or run tests, skip building
+                            Sets the build configuration type
+  -t, --tests               Build and run tests
+  -i, --init                Initialize submodules
+  -v, --version             Show the version number and exit
+  -s, --skip-build          Only do packaging or run tests, skip building
+  --enable-crash-reports    Enable crash reports
 
 positional arguments:
   {pack}
@@ -28,7 +29,7 @@ mutually exclusive required arguments:
   -z, --zip         Build portable archive
 
 examples:
-1.  build.py --qt=C:/Qt/6.4.3/msvc2019_64  # Build the app using a specific Qt
+1.  build.py --qt=C:/Qt/6.6.2/msvc2019_64  # Build the app using a specific Qt
 2.  build.py --init pack --msi             # Build the app and an MSI installer
 3.  build.py --init --tests                # Build the app and run tests
     build.py pack --zip --skip-build       # Generate a 7z archive of the app
@@ -73,7 +74,12 @@ def get_latest_toolset_version():
     if vs_ver is None:
         return None
     vs_ver = int(vs_ver.split(".")[0])
-    return "v143" if vs_ver >= 16 else "v142" if vs_ver >= 15 else "v141"
+    if vs_ver == 17:
+        return "v143"
+    elif vs_ver == 16:
+        return "v142"
+    else:
+        return "v141"
 
 def find_latest_qt_path():
     """Find the latest Qt installation path."""
@@ -205,6 +211,8 @@ def init_submodules():
         "3rdparty/SortFilterProxyModel",
         "3rdparty/md4c",
         "3rdparty/tidy-html5",
+        "3rdparty/zxing-cpp",
+        "3rdparty/hunspell",
     ]
     if execute_cmd(["git", "submodule", "update", "--init" ] + submodules,
                    False):
@@ -255,7 +263,7 @@ def cmake_build(config_str, env_vars, cmake_build_dir):
     return True
 
 
-def build(config_str, qt_dir, tests):
+def build(config_str, qt_dir, tests, build_version, enable_crash_reports, crash_report_url=None):
     """Use cmake to build the project."""
     print("Building with Qt at " + qt_dir)
 
@@ -275,10 +283,20 @@ def build(config_str, qt_dir, tests):
         "-DCMAKE_INSTALL_PREFIX=" + os.getcwd(),
         "-DCMAKE_SYSTEM_VERSION=" + WIN_SDK_VERSION,
         "-DCMAKE_BUILD_TYPE=" + "Release",
-        "-DENABLE_TESTS=" + str(tests).lower(),
+        "-DBUILD_TESTING=" + str(tests).lower(),
         "-DBETA=" + str((0, 1)[config_str == "Beta"]),
     ]
 
+    if enable_crash_reports:
+        cmake_options.append("-DENABLE_CRASHREPORTS=ON")
+        if crash_report_url:
+            cmake_options.append(f"-DCRASH_REPORT_URL={crash_report_url}")
+    else:
+        cmake_options.append("-DENABLE_CRASHREPORTS=OFF")
+
+    if build_version:
+        cmake_options.append("-DBUILD_VERSION=" + build_version)
+
     # Make sure the build directory exists.
     if not os.path.exists(build_dir):
         os.makedirs(build_dir)
@@ -292,11 +310,11 @@ def build(config_str, qt_dir, tests):
         sys.exit(1)
 
 
-def deploy_runtimes(config_str, qt_dir):
+def deploy_runtimes(qt_dir):
     """Deploy the dependencies to the runtime directory."""
     print("Deploying runtime dependencies")
 
-    runtime_dir = os.path.join(repo_root_dir, "x64", config_str)
+    runtime_dir = os.path.join(repo_root_dir, "x64", "Release")
     stamp_file = os.path.join(runtime_dir, ".deploy.stamp")
     if os.path.exists(stamp_file):
         return
@@ -309,8 +327,8 @@ def deploy_runtimes(config_str, qt_dir):
         shutil.copy(os.path.join(rel_path, src), runtime_dir)
 
     print("Copying libjami dependencies")
-    install_file("contrib/build/openssl/libcrypto-1_1-x64.dll", daemon_dir)
-    install_file("contrib/build/openssl/libssl-1_1-x64.dll", daemon_dir)
+    install_file("contrib/build/openssl/libcrypto-3-x64.dll", daemon_dir)
+    install_file("contrib/build/openssl/libssl-3-x64.dll", daemon_dir)
     # Ringtone files (ul,ogg,wav,opus files in the daemon ringtone dir).
 
     print("Copying ringtones")
@@ -336,8 +354,8 @@ def deploy_runtimes(config_str, qt_dir):
     install_file("resources/images/jami.ico", repo_root_dir)
 
     # windeployqt
-    print("Running windeployqt (this may take a while)...")
     win_deploy_qt = os.path.join(qt_dir, "bin", "windeployqt.exe")
+    print(f"Running windeployqt ({win_deploy_qt}) (this may take a while)...")
     qml_src_dir = os.path.join(repo_root_dir, "src", "app")
     installation_dir = get_vs_prop("installationPath")
     if not installation_dir:
@@ -455,6 +473,8 @@ def parse_args():
                         help='Sets the Qt root path')
     parser.add_argument(
         "-a", "--arch", default="x64", help="Sets the build architecture")
+    parser.add_argument(
+        "--build-version", help="Sets the build version string used for defining app build version")
     parser.add_argument(
         "-t", "--tests", action="store_true", help="Build and run tests")
     parser.add_argument(
@@ -466,17 +486,24 @@ def parse_args():
     parser.add_argument(
         "-i", "--init", action="store_true", help="Initialize submodules")
     parser.add_argument(
-        '-sd',
         '--skip-deploy',
         action='store_true',
         default=False,
         help='Force skip deployment of runtime files needed for packaging')
     parser.add_argument(
-        "-sb",
         "--skip-build",
         action="store_true",
         default=False,
         help="Only do packaging or run tests, skip build step")
+    parser.add_argument(
+        '--enable-crash-reports',
+        action='store_true',
+        default=False,
+        help='Enable crash reporting')
+    parser.add_argument(
+        '--crash-report-url',
+        help='Override the crash report submission URL',
+        default=None)
 
     pack_arg_parser = subparsers.add_parser("pack")
     pack_group = pack_arg_parser.add_mutually_exclusive_group(required=True)
@@ -529,9 +556,12 @@ def main():
 
     def do_build(do_tests):
         if not parsed_args.skip_build:
-            build(config_str, parsed_args.qt, do_tests)
+            build(config_str, parsed_args.qt, do_tests,
+                  parsed_args.build_version,
+                  parsed_args.enable_crash_reports,
+                  parsed_args.crash_report_url)
         if not parsed_args.skip_deploy:
-            deploy_runtimes(config_str, parsed_args.qt)
+            deploy_runtimes(parsed_args.qt)
 
     if parsed_args.subcommand == "pack":
         do_build(False)
