@@ -1,12 +1,5 @@
 /*
- * Copyright (C) 2020-2023 Savoir-faire Linux Inc.
- * Author: Edric Ladent Milaret <edric.ladent-milaret@savoirfairelinux.com>
- * Author: Anthony Léonard <anthony.leonard@savoirfairelinux.com>
- * Author: Olivier Soldano <olivier.soldano@savoirfairelinux.com>
- * Author: Andreas Traczyk <andreas.traczyk@savoirfairelinux.com>
- * Author: Isa Nanic <isa.nanic@savoirfairelinux.com>
- * Author: Mingrui Zhang <mingrui.zhang@savoirfairelinux.com>
- * Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
+ * Copyright (C) 2020-2025 Savoir-faire Linux Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,31 +18,34 @@
 #include "calladapter.h"
 
 #include "systemtray.h"
-#include "utils.h"
-#include "qmlregister.h"
-
-#include <QApplication>
-#include <QTimer>
-#include <QJsonObject>
+#include "appsettingsmanager.h"
+#include "pttlistener.h"
 
 #include <api/callmodel.h>
 #include <api/callparticipantsmodel.h>
 
 #include <media_const.h>
 
-CallAdapter::CallAdapter(SystemTray* systemTray, LRCInstance* instance, QObject* parent)
+#include <QApplication>
+#include <QTimer>
+#include <QJsonObject>
+
+CallAdapter::CallAdapter(AppSettingsManager* settingsManager,
+                         SystemTray* systemTray,
+                         LRCInstance* instance,
+                         QObject* parent)
     : QmlAdapterBase(instance, parent)
     , systemTray_(systemTray)
     , callInformationListModel_(std::make_unique<CallInformationListModel>())
 {
+    // Get the PTTListener instance.
+    listener_ = qApp->property("PTTListener").value<PTTListener*>();
+
     set_callInformationList(QVariant::fromValue(callInformationListModel_.get()));
 
     timer = new QTimer(this);
     connect(timer, &QTimer::timeout, this, &CallAdapter::updateAdvancedInformation);
 
-    overlayModel_.reset(new CallOverlayModel(lrcInstance_, this));
-    QML_REGISTERSINGLETONTYPE_POBJECT(NS_MODELS, overlayModel_.get(), "CallOverlayModel");
-
     accountId_ = lrcInstance_->get_currentAccountId();
     connectCallModel(accountId_);
 
@@ -71,7 +67,7 @@ CallAdapter::CallAdapter(SystemTray* systemTray, LRCInstance* instance, QObject*
 #ifdef Q_OS_LINUX
     // notification responses (gnu/linux currently)
     connect(systemTray_,
-            &SystemTray::answerCallActivated,
+            &SystemTray::acceptCallActivated,
             this,
             [this](const QString& accountId, const QString& convUid) {
                 acceptACall(accountId, convUid);
@@ -97,6 +93,65 @@ CallAdapter::CallAdapter(SystemTray* systemTray, LRCInstance* instance, QObject*
             &LRCInstance::selectedConvUidChanged,
             this,
             &CallAdapter::saveConferenceSubcalls);
+
+#ifdef HAVE_GLOBAL_PTT
+    connectPtt();
+#endif
+}
+
+CallAdapter::~CallAdapter()
+{
+#ifdef HAVE_GLOBAL_PTT
+    disconnectPtt();
+#endif
+}
+
+void
+CallAdapter::connectPtt()
+{
+#ifdef HAVE_GLOBAL_PTT
+    if (listener_->getPttState()) {
+        QObject::connect(
+            listener_,
+            &PTTListener::pttKeyPressed,
+            this,
+            [this]() {
+                const auto callId
+                    = lrcInstance_->getCallIdForConversationUid(lrcInstance_->get_selectedConvUid(),
+                                                                accountId_);
+                try {
+                    isMicrophoneMuted_ = isMuted(callId);
+                    if (isMicrophoneMuted_)
+                        muteAudioToggle();
+                } catch (const std::exception& e) {
+                    qWarning() << e.what();
+                }
+            },
+            Qt::QueuedConnection);
+
+        QObject::connect(
+            listener_,
+            &PTTListener::pttKeyReleased,
+            this,
+            [this]() {
+                if (isMicrophoneMuted_) {
+                    muteAudioToggle();
+                }
+            },
+            Qt::QueuedConnection);
+    }
+#endif
+}
+
+void
+CallAdapter::disconnectPtt()
+{
+#ifdef HAVE_GLOBAL_PTT
+    if (listener_->getPttState()) {
+        QObject::disconnect(listener_, &PTTListener::pttKeyPressed, this, nullptr);
+        QObject::disconnect(listener_, &PTTListener::pttKeyReleased, this, nullptr);
+    }
+#endif
 }
 
 void
@@ -172,6 +227,12 @@ CallAdapter::onCallStarted(const QString& callId)
     // update call Information list by adding the new information related to the callId
     callInformationListModel_->addElement(
         qMakePair(callId, callModel->advancedInformationForCallId(callId)));
+    if (listener_->getPttState()) {
+#ifdef HAVE_GLOBAL_PTT
+        listener_->startListening();
+        toMute += callId;
+#endif
+    }
 }
 
 void
@@ -181,10 +242,14 @@ CallAdapter::onCallEnded(const QString& callId)
         return;
     // update call Information list by removing information related to the callId
     callInformationListModel_->removeElement(callId);
+#ifdef HAVE_GLOBAL_PTT
+    if (listener_->getPttState() && !hasCall_)
+        listener_->stopListening();
+#endif
 }
 
 void
-CallAdapter::onCallStatusChanged(const QString& callId, int code)
+CallAdapter::onCallStatusChanged(const QString& accountId, const QString& callId, int code)
 {
     Q_UNUSED(code)
 
@@ -237,7 +302,7 @@ CallAdapter::onCallStatusChanged(const QString& callId, int code)
                     currentConfSubcalls_.clear();
                     return;
                 }
-            } else {
+            } else if (currentConfSubcalls_.size() >= 2) {
                 // okay, still a conference, so just update the subcall list and this call
                 saveConferenceSubcalls();
                 Q_EMIT lrcInstance_->conversationUpdated(currentConvInfo.uid, accountId_);
@@ -272,10 +337,20 @@ CallAdapter::onCallStatusChanged(const QString& callId, int code)
 }
 
 void
-CallAdapter::onCallAddedToConference(const QString& callId, const QString& confId)
+CallAdapter::onCallInfosChanged(const QString& accountId, const QString& callId)
+{
+    Q_UNUSED(accountId)
+    auto mute = toMute.remove(callId);
+    if (mute && listener_->getPttState())
+        muteAudioToggle();
+}
+
+void
+CallAdapter::onCallAddedToConference(const QString& callId, const QString& conversationId, const QString& confId)
 {
     Q_UNUSED(callId)
     Q_UNUSED(confId)
+    Q_UNUSED(conversationId)
     saveConferenceSubcalls();
 }
 
@@ -307,16 +382,13 @@ CallAdapter::hangUpACall(const QString& accountId, const QString& convUid)
 }
 
 void
-CallAdapter::setCallMedia(const QString& accountId, const QString& convUid, bool video)
+CallAdapter::setCallMedia(const QString& accountId, const QString& convUid, bool videoMuted)
 {
     const auto& convInfo = lrcInstance_->getConversationFromConvUid(convUid, accountId);
     if (convInfo.uid.isEmpty())
         return;
-    try {
-        lrcInstance_->getAccountInfo(accountId).callModel->updateCallMediaList(convInfo.callId,
-                                                                               video);
-    } catch (...) {
-    }
+
+    lrcInstance_->getAccountInfo(accountId).callModel->setVideoMuted(convInfo.callId, videoMuted);
 }
 
 void
@@ -396,11 +468,14 @@ CallAdapter::onShowIncomingCallView(const QString& accountId, const QString& con
             showNotification(accountId, convInfo.uid);
             return;
         }
+        if (!accountProperties.denySecondCall) {
+            lrcInstance_->selectConversation(convInfo.uid, accountId);
+        }
+    } else {
+        // finally, in this case, the conversation isn't selected yet
+        // and there are no other special conditions, so just select the conversation
+        lrcInstance_->selectConversation(convInfo.uid, accountId);
     }
-
-    // finally, in this case, the conversation isn't selected yet
-    // and there are no other special conditions, so just select the conversation
-    lrcInstance_->selectConversation(convInfo.uid, accountId);
 }
 
 void
@@ -491,7 +566,13 @@ CallAdapter::connectCallModel(const QString& accountId)
     connect(accInfo.callModel.get(),
             &CallModel::callStatusChanged,
             this,
-            QOverload<const QString&, int>::of(&CallAdapter::onCallStatusChanged),
+            QOverload<const QString&, const QString&, int>::of(&CallAdapter::onCallStatusChanged),
+            Qt::UniqueConnection);
+
+    connect(accInfo.callModel.get(),
+            &CallModel::callInfosChanged,
+            this,
+            &CallAdapter::onCallInfosChanged,
             Qt::UniqueConnection);
 
     connect(accInfo.callModel.get(),
@@ -816,6 +897,23 @@ CallAdapter::holdThisCallToggle()
     }
 }
 
+bool
+CallAdapter::isMuted(const QString& callId)
+{
+    if (!(callId.isEmpty() || !lrcInstance_->getCurrentCallModel()->hasCall(callId))) {
+        auto* callModel = lrcInstance_->getCurrentCallModel();
+        if (callModel->hasCall(callId)) {
+            const auto callInfo = lrcInstance_->getCurrentCallModel()->getCall(callId);
+            auto mute = false;
+            for (const auto& m : callInfo.mediaList)
+                if (m[libjami::Media::MediaAttributeKey::LABEL] == "audio_0")
+                    mute = m[libjami::Media::MediaAttributeKey::MUTED] == TRUE_STR;
+            return mute;
+        }
+    }
+    throw std::runtime_error("CallAdapter::isMuted: callId is empty or call does not exist");
+}
+
 void
 CallAdapter::muteAudioToggle()
 {
@@ -825,13 +923,10 @@ CallAdapter::muteAudioToggle()
         return;
     }
     auto* callModel = lrcInstance_->getCurrentCallModel();
-    if (callModel->hasCall(callId)) {
-        const auto callInfo = lrcInstance_->getCurrentCallModel()->getCall(callId);
-        auto mute = false;
-        for (const auto& m : callInfo.mediaList)
-            if (m[libjami::Media::MediaAttributeKey::LABEL] == "audio_0")
-                mute = m[libjami::Media::MediaAttributeKey::MUTED] == FALSE_STR;
-        callModel->muteMedia(callId, "audio_0", mute);
+    try {
+        callModel->muteMedia(callId, "audio_0", !isMuted(callId));
+    } catch (const std::exception& e) {
+        qWarning() << e.what();
     }
 }
 
@@ -877,7 +972,8 @@ CallAdapter::muteCameraToggle()
             callModel->removeMedia(callId,
                                    libjami::Media::Details::MEDIA_TYPE_VIDEO,
                                    libjami::Media::VideoProtocolPrefix::CAMERA,
-                                   mute);
+                                   mute,
+                                   false);
         else
             callModel->addMedia(callId,
                                 lrcInstance_->avModel().getCurrentVideoCaptureDevice(),
