@@ -1,8 +1,5 @@
 /*
- * Copyright (C) 2019-2023 Savoir-faire Linux Inc.
- * Author: Andreas Traczyk <andreas.traczyk@savoirfairelinux.com>
- * Author: Isa Nanic <isa.nanic@savoirfairelinux.com>
- * Author: Mingrui Zhang <mingrui.zhang@savoirfairelinux.com>
+ * Copyright (C) 2019-2025 Savoir-faire Linux Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +17,9 @@
 
 #include "lrcinstance.h"
 
+#include "global.h"
+#include "connectivitymonitor.h"
+
 #include <QBuffer>
 #include <QMutex>
 #include <QObject>
@@ -27,21 +27,21 @@
 #include <QRegularExpression>
 #include <QtConcurrent/QtConcurrent>
 
-LRCInstance::LRCInstance(migrateCallback willMigrateCb,
-                         migrateCallback didMigrateCb,
-                         const QString& updateUrl,
+LRCInstance::LRCInstance(const QString& updateUrl,
                          ConnectivityMonitor* connectivityMonitor,
-                         bool muteDring)
-    : lrc_(std::make_unique<Lrc>(willMigrateCb, didMigrateCb, muteDring))
-    , updateManager_(std::make_unique<UpdateManager>(updateUrl, connectivityMonitor, this))
+                         bool debugMode,
+                         bool muteDaemon)
+    : lrc_(std::make_unique<Lrc>(!debugMode || muteDaemon))
+    , updateManager_(std::make_unique<AppVersionManager>(updateUrl, connectivityMonitor, this))
+    , connectivityMonitor_(*connectivityMonitor)
     , threadPool_(new QThreadPool(this))
 {
-    debugMode_ = !muteDring;
+    debugMode_ = debugMode;
+    muteDaemon_ = muteDaemon;
     threadPool_->setMaxThreadCount(1);
 
-    connect(this, &LRCInstance::currentAccountIdChanged, [this] {
-        // save to config, editing the accountlistmodel's underlying data
-        accountModel().setTopAccount(currentAccountId_);
+    // Update the current account when the account list changes.
+    connect(&accountModel(), &AccountModel::accountsReordered, this, [this] {
         Q_EMIT accountListChanged();
 
         profile::Info profileInfo;
@@ -58,6 +58,11 @@ LRCInstance::LRCInstance(migrateCallback willMigrateCb,
         set_currentAccountAvatarSet(!profileInfo.avatar.isEmpty());
     });
 
+    connect(this, &LRCInstance::currentAccountIdChanged, [this] {
+        // This will trigger `AccountModel::accountsReordered`.
+        accountModel().setTopAccount(currentAccountId_);
+    });
+
     connect(&accountModel(), &AccountModel::profileUpdated, this, [this](const QString& id) {
         if (id != currentAccountId_)
             return;
@@ -66,6 +71,12 @@ LRCInstance::LRCInstance(migrateCallback willMigrateCb,
         set_currentAccountAvatarSet(!getCurrentAccountInfo().profileInfo.avatar.isEmpty());
     });
 
+    connect(&accountModel(),
+            &AccountModel::accountRemoved,
+            this,
+            &LRCInstance::onAccountRemoved,
+            Qt::DirectConnection);
+
     // set the current account if any
     auto accountList = accountModel().getAccountList();
     if (accountList.size()) {
@@ -73,8 +84,8 @@ LRCInstance::LRCInstance(migrateCallback willMigrateCb,
     }
 };
 
-UpdateManager*
-LRCInstance::getUpdateManager()
+AppVersionManager*
+LRCInstance::getAppVersionManager()
 {
     return updateManager_.get();
 }
@@ -109,6 +120,12 @@ LRCInstance::pluginModel()
     return lrc_->getPluginModel();
 }
 
+ConnectivityMonitor&
+LRCInstance::connectivityMonitor()
+{
+    return connectivityMonitor_;
+}
+
 bool
 LRCInstance::isConnected()
 {
@@ -266,7 +283,7 @@ LRCInstance::getCurrentContactModel()
 int
 LRCInstance::getCurrentAccountIndex()
 {
-    for (int i = 0; i < accountModel().getAccountList().size(); i++) {
+    for (int i = 0; i < accountModel().getAccountCount(); i++) {
         if (accountModel().getAccountList()[i] == get_currentAccountId()) {
             return i;
         }
@@ -332,30 +349,36 @@ LRCInstance::stopAudioMeter()
     });
 }
 
-QString
+QVariantMap
 LRCInstance::getContentDraft(const QString& convUid, const QString& accountId)
 {
     auto draftKey = accountId + "_" + convUid;
-    return contentDrafts_[draftKey];
+    QVariantMap draftMap;
+    draftMap["text"] = contentDrafts_[draftKey];
+    draftMap["files"] = fileDrafts_[draftKey];
+
+    return draftMap;
 }
 
 void
 LRCInstance::setContentDraft(const QString& convUid,
                              const QString& accountId,
-                             const QString& content)
+                             const QString& textDraft,
+                             const QList<QString>& filePathDraft)
 {
     if (accountId.isEmpty() || convUid.isEmpty()) {
         return;
     }
-
     auto draftKey = accountId + "_" + convUid;
 
     // prevent a senseless dataChanged signal from the
     // model if nothing has changed
-    if (contentDrafts_[draftKey] == content)
+    if (contentDrafts_[draftKey] == textDraft && fileDrafts_[draftKey] == filePathDraft) {
         return;
+    }
 
-    contentDrafts_[draftKey] = content;
+    contentDrafts_[draftKey] = textDraft;
+    fileDrafts_[draftKey] = filePathDraft;
     // this signal is only needed to update the current smartlist
     Q_EMIT draftSaved(convUid);
 }
@@ -386,7 +409,7 @@ LRCInstance::indexOfActiveCall(const QString& confId, const QString& uri, const
 {
     if (auto optConv = getCurrentConversationModel()->getConversationForUid(selectedConvUid_)) {
         auto& convInfo = optConv->get();
-        return convInfo.indexOfActiveCall({{"confId", confId}, {"uri", uri}, {"device", deviceId}});
+        return convInfo.indexOfActiveCall(confId, uri, deviceId);
     }
     return -1;
 }
@@ -394,6 +417,10 @@ LRCInstance::indexOfActiveCall(const QString& confId, const QString& uri, const
 void
 LRCInstance::deselectConversation()
 {
+    // Only do this if we have an account selected
+    if (get_currentAccountId().isEmpty()) {
+        return;
+    }
     set_selectedConvUid();
 }
 
@@ -448,3 +475,33 @@ LRCInstance::set_selectedConvUid(QString selectedConvUid)
         Q_EMIT selectedConvUidChanged();
     }
 }
+
+VectorMapStringString
+LRCInstance::getConnectionList(const QString& accountId, const QString& uid)
+{
+    return Lrc::getConnectionList(accountId, uid);
+}
+
+VectorMapStringString
+LRCInstance::getChannelList(const QString& accountId, const QString& uid)
+{
+    return Lrc::getChannelList(accountId, uid);
+}
+
+void
+LRCInstance::onAccountRemoved(const QString& accountId)
+{
+    if (accountId != currentAccountId_)
+        return;
+
+    // If there are any accounts left, select the first one, otherwise clear the current account
+    // and request presentation of the wizard view.
+    auto accountList = accountModel().getAccountList();
+    if (accountList.size()) {
+        set_currentAccountId(accountList.at(0));
+    } else {
+        set_currentAccountId();
+    }
+
+    Q_EMIT currentAccountRemoved();
+}
