@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2022 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2025 Savoir-faire Linux Inc.
  *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
  *          Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *          Vsevolod Ivanov <vsevolod.ivanov@savoirfairelinux.com>
@@ -58,6 +58,19 @@ constexpr const std::chrono::minutes PRINT_STATS_PERIOD {2};
 using ResponseByParts = restinio::chunked_output_t;
 using ResponseByPartsBuilder = restinio::response_builder_t<ResponseByParts>;
 
+namespace http {
+/**
+ * Session value associated with a connection_id_t key.
+ */
+struct ListenerSession
+{
+    ListenerSession() = default;
+    dht::InfoHash hash;
+    std::future<size_t> token;
+    std::shared_ptr<restinio::response_builder_t<restinio::chunked_output_t>> response;
+};
+}
+
 class opendht_logger_t
 {
 public:
@@ -87,6 +100,47 @@ private:
     std::shared_ptr<Logger> m_logger;
 };
 
+
+std::string
+DhtProxyServer::ServerStats::toString() const
+{
+    auto ret = fmt::format("Listens: {}, Puts: {}, PushListeners: {}\n"
+                "Push requests in the last {}: [Android: {}], [iOS: {}], [Unified: {}]\n"
+                "Requests: {} per second.",
+                listenCount, putCount, pushListenersCount,
+                print_duration(lastUpdated - serverStartTime),
+                androidPush.toString(), iosPush.toString(), unifiedPush.toString(),
+                requestRate);
+    if (nodeInfo) {
+        auto& ipv4 = nodeInfo->ipv4;
+        if (ipv4.table_depth > 1)
+            ret += fmt::format("IPv4 Network estimation: {}\n", ipv4.getNetworkSizeEstimation());
+        auto& ipv6 = nodeInfo->ipv6;
+        if (ipv6.table_depth > 1)
+            ret += fmt::format("IPv6 Network estimation: {}\n", ipv6.getNetworkSizeEstimation());
+    }
+    return ret;
+}
+
+Json::Value
+DhtProxyServer::ServerStats::toJson() const
+{
+    Json::Value result;
+    result["listenCount"] = static_cast<Json::UInt64>(listenCount);
+    result["putCount"] = static_cast<Json::UInt64>(putCount);
+    result["totalPermanentPuts"] = static_cast<Json::UInt64>(totalPermanentPuts);
+    result["pushListenersCount"] = static_cast<Json::UInt64>(pushListenersCount);
+    result["serverStartTime"] = static_cast<Json::LargestInt>(to_time_t(serverStartTime));
+    result["lastUpdated"] = static_cast<Json::LargestInt>(to_time_t(lastUpdated));
+    result["androidPush"] = androidPush.toJson();
+    result["iosPush"] = iosPush.toJson();
+    result["unifiedPush"] = unifiedPush.toJson();
+    result["requestRate"] = requestRate;
+    if (nodeInfo)
+        result["nodeInfo"] = nodeInfo->toJson();
+    return result;
+}
+
 restinio::request_handling_status_t
 DhtProxyServer::serverError(restinio::request_t& request) {
     auto response = initHttpResponse(request.create_response(restinio::status_internal_server_error()));
@@ -116,7 +170,7 @@ private:
 void
 DhtProxyServer::ConnectionListener::state_changed(const restinio::connection_state::notice_t& notice) noexcept
 {
-    if (restinio::holds_alternative<restinio::connection_state::closed_t>(notice.cause())) {
+    if (std::holds_alternative<restinio::connection_state::closed_t>(notice.cause())) {
         onClosed_(notice.connection_id());
     }
 }
@@ -210,6 +264,7 @@ DhtProxyServer::DhtProxyServer(const std::shared_ptr<DhtRunner>& dht,
     :   ioContext_(std::make_shared<asio::io_context>()),
         dht_(dht), persistPath_(config.persistStatePath), logger_(logger),
         printStatsTimer_(std::make_unique<asio::steady_timer>(*ioContext_, 3s)),
+        serverStartTime_(clock::now()),
         connListener_(std::make_shared<ConnectionListener>(std::bind(&DhtProxyServer::onConnectionClosed, this, std::placeholders::_1))),
         pushServer_(config.pushServer),
         bundleId_(config.bundleId)
@@ -402,31 +457,12 @@ DhtProxyServer::loadState(Is& is, size_t size) {
                 for (auto& pushListener : pushListeners_) {
                     for (auto& listeners : pushListener.second.listeners) {
                         for (auto& listener : listeners.second) {
+                            // start listening
                             listener.internalToken = dht_->listen(listeners.first,
-                                [this, infoHash=listeners.first, pushToken=pushListener.first, type=listener.type, clientId=listener.clientId, sessionCtx = listener.sessionCtx, topic=listener.topic]
-                                (const std::vector<std::shared_ptr<Value>>& values, bool expired) {
-                                    // Build message content
-                                    Json::Value json;
-                                    json["key"] = infoHash.toString();
-                                    json["to"] = clientId;
-                                    json["t"] = Json::Value::Int64(std::chrono::duration_cast<std::chrono::milliseconds>(system_clock::now().time_since_epoch()).count());
-                                    {
-                                        std::lock_guard<std::mutex> l(sessionCtx->lock);
-                                        json["s"] = sessionCtx->sessionId;
-                                    }
-                                    if (expired and values.size() < 2){
-                                        std::ostringstream ss;
-                                        for(size_t i = 0; i < values.size(); ++i){
-                                            if(i != 0) ss << ",";
-                                            ss << values[i]->id;
-                                        }
-                                        json["exp"] = ss.str();
-                                    }
-                                    auto maxPrio = 1000u;
-                                    for (const auto& v : values)
-                                        maxPrio = std::min(maxPrio, v->priority);
-                                    sendPushNotification(pushToken, std::move(json), type, !expired and maxPrio == 0, topic);
-                                    return true;
+                                [this, infoHash=listeners.first, pushToken=pushListener.first, type=listener.type,
+                                 clientId=listener.clientId, sessionCtx = listener.sessionCtx, topic=listener.topic]
+                                (const std::vector<Sp<Value>>& values, bool expired) {
+                                    return this->handlePushListen(infoHash, pushToken, type, clientId, sessionCtx, topic, values, expired);
                                 }
                             );
                             // expire notify
@@ -548,7 +584,15 @@ DhtProxyServer::updateStats(std::shared_ptr<NodeInfo> info) const
     stats.requestRate = count / dt.count();
 #ifdef OPENDHT_PUSH_NOTIFICATIONS
     stats.pushListenersCount = pushListeners_.size();
+    {
+        std::lock_guard lk(pushStatsMutex_);
+        stats.androidPush = androidPush_;
+        stats.iosPush = iosPush_;
+        stats.unifiedPush = unifiedPush_;
+    }
 #endif
+    stats.serverStartTime = serverStartTime_;
+    stats.lastUpdated = now;
     stats.totalPermanentPuts = 0;
     std::for_each(puts_.begin(), puts_.end(), [&stats](const auto& put) {
         stats.totalPermanentPuts += put.second.puts.size();
@@ -628,6 +672,8 @@ DhtProxyServer::createRestRouter()
     router->http_get("/key/:hash", std::bind(&DhtProxyServer::get, this, _1, _2));
     // key.post
     router->http_post("/key/:hash", std::bind(&DhtProxyServer::put, this, _1, _2));
+    router->add_handler(restinio::http_method_options(),
+                        "/key/:hash/listen", std::bind(&DhtProxyServer::options, this, _1, _2));
     // key.listen
     router->http_get("/key/:hash/listen", std::bind(&DhtProxyServer::listen, this, _1, _2));
 #ifdef OPENDHT_PUSH_NOTIFICATIONS
@@ -897,13 +943,13 @@ DhtProxyServer::subscribe(restinio::request_handle_t request,
         // Send response
         if (not newListener) {
             if (logger_)
-                logger_->d("[proxy:server] [subscribe] found [client %s]", listener.clientId.c_str());
+                logger_->d("[proxy:server] [subscribe %s] found [client %s]", infoHash.toString().c_str(), listener.clientId.c_str());
             // Send response header
             auto response = std::make_shared<ResponseByPartsBuilder>(initHttpResponse(request->create_response<ResponseByParts>()));
             response->flush();
             if (!root["refresh"].asBool()) {
                 // No Refresh
-                dht_->get(infoHash, [this, response](const Sp<Value>& value){
+                dht_->get(infoHash, [this, response](const Sp<Value>& value) {
                     auto output = Json::writeString(jsonBuilder_, value->toJson()) + "\n";
                     response->append_chunk(output);
                     response->flush();
@@ -920,33 +966,15 @@ DhtProxyServer::subscribe(restinio::request_handle_t request,
         } else {
             // =========== No existing listener for an infoHash ============
             // Add listen on dht
+            if (logger_)
+                logger_->d("[proxy:server] [subscribe %s] new", infoHash.toString().c_str());
             listener.internalToken = dht_->listen(infoHash,
                 [this, infoHash, pushToken, type, clientId, sessionCtx = listener.sessionCtx, topic]
-                (const std::vector<std::shared_ptr<Value>>& values, bool expired){
-                    // Build message content
-                    Json::Value json;
-                    json["key"] = infoHash.toString();
-                    json["to"] = clientId;
-                    json["t"] = Json::Value::Int64(std::chrono::duration_cast<std::chrono::milliseconds>(system_clock::now().time_since_epoch()).count());
-                    {
-                        std::lock_guard<std::mutex> l(sessionCtx->lock);
-                        json["s"] = sessionCtx->sessionId;
-                    }
-                    if (expired and values.size() < 2){
-                        std::ostringstream ss;
-                        for(size_t i = 0; i < values.size(); ++i) {
-                            if(i != 0) ss << ",";
-                            ss << values[i]->id;
-                        }
-                        json["exp"] = ss.str();
-                    }
-                    auto maxPrio = 1000u;
-                    for (const auto& v : values)
-                        maxPrio = std::min(maxPrio, v->priority);
-                    sendPushNotification(pushToken, std::move(json), type, !expired and maxPrio == 0, topic);
-                    return true;
+                (const std::vector<Sp<Value>>& values, bool expired) {
+                    return this->handlePushListen(infoHash, pushToken, type, clientId, sessionCtx, topic, values, expired);
                 }
             );
+            // Send response header
             auto response = initHttpResponse(request->create_response());
             response.set_body("{}\n");
             return response.done();
@@ -1050,10 +1078,54 @@ DhtProxyServer::handleCancelPushListen(const asio::error_code &ec, const std::st
         pushListeners_.erase(pushListener);
 }
 
+bool
+DhtProxyServer::handlePushListen(const InfoHash& infoHash, const std::string& pushToken, PushType type, const std::string& clientId,
+                                 const std::shared_ptr<DhtProxyServer::PushSessionContext>& sessionCtx, const std::string& topic,
+                                 const std::vector<std::shared_ptr<Value>>& values, bool expired)
+{
+    Json::Value json;
+    json["key"] = infoHash.toString();
+    json["to"] = clientId;
+    using namespace std::chrono;
+    json["t"] = Json::Value::Int64(duration_cast<milliseconds>(system_clock::now().time_since_epoch()).count());
+
+    {
+        std::lock_guard<std::mutex> l(sessionCtx->lock);
+        json["s"] = sessionCtx->sessionId;
+    }
+
+    // Build a comma-separated list of ids from the values. This is intended to be used when
+    // streaming from the iOS notification extension in order to filter out unwanted values.
+    std::string ids;
+    for(const auto& value : values) {
+        if (!ids.empty()) ids += ",";
+        ids += std::to_string(value->id);
+    }
+    json["ids"] = ids;
+
+    // If message is expired copy the value ID to the `exp` field.
+    // This is presumably used by the Android notification system.
+    if (expired && values.size() < 2) {
+        json["exp"] = json["ids"];
+    }
+
+    auto minPriority = 1000u;
+    for (const auto& v : values)
+        minPriority = std::min(minPriority, v->priority);
+
+    if (logger_)
+        logger_->d("[proxy:server] [listen %s] [client %s] [session %s] [expired %i] [priority %i] [values %zu]",
+                   infoHash.toString().c_str(), clientId.c_str(), sessionCtx->sessionId.c_str(), expired, minPriority, values.size());
+
+    sendPushNotification(pushToken, std::move(json), type, !expired and minPriority == 0, topic);
+
+    return true;
+}
+
 void
 DhtProxyServer::sendPushNotification(const std::string& token, Json::Value&& json, PushType type, bool highPriority, const std::string& topic)
 {
-    if (pushServer_.empty())
+    if (pushServer_.empty() && type != PushType::UnifiedPush)
         return;
 
     unsigned reqid = 0;
@@ -1061,7 +1133,7 @@ DhtProxyServer::sendPushNotification(const std::string& token, Json::Value&& jso
         std::shared_ptr<http::Request> request;
         http::Url tokenUrl(token);
         if (type == PushType::UnifiedPush)
-            request = std::make_shared<http::Request>(io_context(), tokenUrl.protocol + "://" + tokenUrl.host, logger_);
+            request = std::make_shared<http::Request>(io_context(), tokenUrl.protocol + "://" + tokenUrl.host + (tokenUrl.service.empty() ? "" : (":" + tokenUrl.service)), logger_);
         else
             request = std::make_shared<http::Request>(io_context(), pushHostPort_.first, pushHostPort_.second, pushHostPort_.first.find("https://") == 0, logger_);;
         reqid = request->id();
@@ -1080,27 +1152,33 @@ DhtProxyServer::sendPushNotification(const std::string& token, Json::Value&& jso
             request->set_body(Json::writeString(jsonBuilder_, std::move(json)));
         } else {
             // NOTE: see https://github.com/appleboy/gorush
+            auto isResubscribe = json.isMember("timeout");
             Json::Value notification(Json::objectValue);
             Json::Value tokens(Json::arrayValue);
             tokens[0] = token;
             notification["tokens"] = std::move(tokens);
             notification["platform"] = type == PushType::Android ? 2 : 1;
             notification["data"] = std::move(json);
-            notification["priority"] = highPriority ? "high" : "normal";
-            if (type == PushType::Android)
-                notification["time_to_live"] = 3600 * 24; // 24 hours
-            else {
+            auto priority = highPriority ? "high" : "normal";
+            if (type == PushType::Android) {
+                Json::Value androidConfig(Json::objectValue);
+                androidConfig["priority"] = priority;
+                androidConfig["ttl"] = "86400s"; // time to live = 24 hours
+                notification["android"] = std::move(androidConfig);
+            } else {
+                notification["priority"] = priority;
                 const auto expiration = std::chrono::system_clock::now() + std::chrono::hours(24);
                 uint32_t exp = std::chrono::duration_cast<std::chrono::seconds>(expiration.time_since_epoch()).count();
                 notification["expiration"] = exp;
                 if (!topic.empty())
                     notification["topic"] = topic;
-                if (highPriority) {
+                if (highPriority || isResubscribe) {
                     Json::Value alert(Json::objectValue);
                     alert["title"]="hello";
                     notification["push_type"] = "alert";
                     notification["alert"] = alert;
                     notification["mutable_content"] = true;
+                    notification["priority"] = "high";
                 } else {
                     notification["push_type"] = "background";
                     notification["content_available"] = true;
@@ -1129,6 +1207,21 @@ DhtProxyServer::sendPushNotification(const std::string& token, Json::Value&& jso
             requests_[reqid] = request;
         }
         request->send();
+        // For monitoring purposes
+        std::lock_guard lk(pushStatsMutex_);
+        switch (type) {
+        case PushType::Android:
+            androidPush_.increment(highPriority);
+            break;
+        case PushType::iOS:
+            iosPush_.increment(highPriority);
+            break;
+        case PushType::UnifiedPush:
+            unifiedPush_.increment(highPriority);
+            break;
+        default:
+            break;
+        }
     }
     catch (const std::exception &e){
         if (logger_)
