@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -1057,11 +1057,7 @@ UPnPContext::onMappingAdded(const std::shared_ptr<IGD>& igd, const Mapping& mapR
     }
 
     // The mapping request is new and successful. Update.
-    map->setIgd(igd);
-    map->setInternalAddress(mapRes.getInternalAddress());
-    map->setExternalPort(mapRes.getExternalPort());
-    map->setRenewalTime(mapRes.getRenewalTime());
-    map->setExpiryTime(mapRes.getExpiryTime());
+    map->setAddedInfo(mapRes);
     // Update the state and report to the owner.
     updateMappingState(map, MappingState::OPEN);
     scheduleMappingsRenewal();
